from policyengine_us.model_api import *


class taxable_social_security_tier_2(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Taxable Social Security (tier 2)"
    documentation = "Taxable Social Security from 50-85% taxation tier, credited to Medicare HI trust fund"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/86#a_2"

    # tier_2 = total_taxable - tier_1 capped at 0
    def formula(tax_unit, period, parameters):
        tier_1 = tax_unit("taxable_social_security_tier_1", period)
        total_taxable = tax_unit("tax_unit_taxable_social_security", period)
        return max_(total_taxable - tier_1, 0)



from policyengine_us.model_api import *


class ga_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Georgia Temporary Assistance for Needy Families (TANF)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://rules.sos.ga.gov/gac/290-2-28",
        "https://pamms.dhs.ga.gov/dfcs/tanf/",
    )
    defined_for = "ga_tanf_eligible"

    def formula(spm_unit, period, parameters):
        standard_of_need = spm_unit("ga_tanf_standard_of_need", period)
        countable_income = spm_unit("ga_tanf_countable_income", period)
        family_maximum = spm_unit("ga_tanf_family_maximum", period)

        # Benefit = MIN(Standard of Need - Countable Income, Family Maximum)
        benefit_before_max = max_(standard_of_need - countable_income, 0)
        return min_(benefit_before_max, family_maximum)



from policyengine_us.model_api import *


class ky_ktap_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Kentucky K-TAP"
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        income_eligible = spm_unit("ky_ktap_income_eligible", period)
        resource_eligible = spm_unit("ky_ktap_resource_eligible", period)
        return demographic_eligible & income_eligible & resource_eligible



from policyengine_us.model_api import *


class ri_works_resource_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Resource eligible for Rhode Island Works"
    definition_period = MONTH
    reference = (
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
        "https://dhs.ri.gov/programs-and-services/ri-works-program/eligibility-how-apply",
    )
    defined_for = StateCode.RI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ri.dhs.works
        resources = spm_unit("spm_unit_assets", period.this_year)
        return resources <= p.resource_limit



from policyengine_us.model_api import *


class az_property_tax_credit_agi(Variable):
    value_type = float
    entity = TaxUnit
    label = "Arizona adjusted gross income for property tax credit"
    unit = USD
    definition_period = YEAR
    documentation = (
        "Adjusted gross income as defined for Arizona property tax credit purposes. "
        "Per ARS 43-1072(H)(6) and ITR 12-1, this starts with Federal AGI and only "
        "excludes items specifically listed in ARS 43-1072(I): Social Security, "
        "railroad retirement, workers compensation, Arizona unemployment, veterans "
        "disability pensions, welfare, and gifts. Unlike regular Arizona income tax, "
        "this does NOT exclude pension income, capital gains, or Arizona exemptions."
    )
    reference = [
        "https://www.azleg.gov/ars/43/01072.htm",  # ARS 43-1072
        "https://azdor.gov/sites/default/files/2023-03/RULINGS_INDV_2012_itr12-1.pdf",  # ITR 12-1
    ]
    defined_for = StateCode.AZ

    def formula(tax_unit, period, parameters):
        # Start with Federal AGI
        # Per ITR 12-1, income includes wages, interest, business/farm income,
        # rent/royalty, S-corp/partnership income, alimony, capital gains,
        # pension/annuity income, and other non-excluded income.
        federal_agi = tax_unit("adjusted_gross_income", period)

        # Per ARS 43-1072(I) and ITR 12-1 "Items Excluded from Income",
        # we ONLY exclude Social Security benefits (and other items like
        # railroad retirement, workers comp, AZ unemployment, veterans
        # disability pensions, welfare, and gifts - but these are typically
        # not in Federal AGI anyway).
        #
        # Federal AGI only includes the TAXABLE portion of Social Security,
        # but for property tax credit, ALL Social Security should be excluded.
        # So we subtract the taxable portion that's in Federal AGI.
        taxable_social_security = tax_unit(
            "tax_unit_taxable_social_security", period
        )

        # NOTE: We do NOT subtract Arizona's regular subtractions here because:
        # - Pension exclusions (az_public_pension_exclusion,
        #   az_military_retirement_subtraction) should be INCLUDED per
        #   ITR 12-1 item (9)
        # - Capital gains subtraction (az_long_term_capital_gains_subtraction)
        #   should be INCLUDED per ITR 12-1 item (7)
        # - US Government interest should be INCLUDED per ITR 12-1 item (2)
        # - Arizona exemptions (aged, blind) should NOT be subtracted

        return max_(0, federal_agi - taxable_social_security)



from policyengine_us.model_api import *


class ms_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Mississippi TANF income eligibility"
    definition_period = MONTH
    defined_for = StateCode.MS
    reference = (
        "https://www.mdhs.ms.gov/wp-content/uploads/2018/02/MDHS_TANF-Eligibility-Flyer.pdf",
        "https://www.law.cornell.edu/regulations/mississippi/Miss-Code-tit-18-pt-19",
        "https://www.mdhs.ms.gov/help/tanf/applying-for-tanf/",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ms.dhs.tanf

        # Use federal baseline for gross income
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )

        # Get need standard based on household size (capped at table max)
        size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(size, p.need_standard.max_table_size).astype(int)
        need_standard = p.need_standard.amount[capped_size]

        # Gross income limit is 185% of need standard
        gross_income_limit = need_standard * p.income.gross_income_limit_rate

        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class ri_section_179_expense_add_back(Variable):
    value_type = float
    entity = TaxUnit
    label = "Rhode Island Section 179 expense add-back"
    unit = USD
    definition_period = YEAR
    documentation = (
        "Rhode Island limits Section 179 depreciation to $25,000. "
        "If the federal return includes additional Section 179 depreciation "
        "under I.R.S. Code 179(b) beyond what was allowed prior to P.L. 119-21, "
        "H.R.1 (2025), that additional amount must be added back to federal AGI "
        "for Rhode Island purposes."
    )
    reference = (
        "https://tax.ri.gov/sites/g/files/xkgbur541/files/2025-12/2025%201040R%20Instructions%20122025.pdf#page=12",
        "http://webserver.rilin.state.ri.us/Statutes/title44/44-30/44-30-12.HTM",
    )
    defined_for = StateCode.RI
    # Other P.L. 119-21, H.R.1 (2025) add-backs NOT modeled (RI Schedule HR1):
    # - Business interest expense deduction [I.R.S. Code 163(j)]
    # - Section 174A Amortization Adjustment for R&E expenditures
    # - Qualified sound recording production deduction [I.R.S. Code 181]



from policyengine_us.model_api import *


class de_tanf_dependent_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Delaware TANF dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://help.workworldapp.com/wwwebhelp/de_earned_income_disregards_tanf_and_ga.htm",
    )
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Per DSSM 4008: Dependent care up to $200/month for under age 2,
        # $175/month for age 2 and older
        p = parameters(period).gov.states.de.dhss.tanf.income.deductions

        person = spm_unit.members
        is_dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)

        # Calculate maximum deduction per dependent based on age
        max_per_dependent = p.dependent_care.calc(age)
        total_max = spm_unit.sum(max_per_dependent * is_dependent)

        # Cap at actual childcare expenses
        childcare_expenses = spm_unit("childcare_expenses", period)

        return min_(childcare_expenses, total_max)



from policyengine_us.model_api import *


class de_tanf_net_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Delaware TANF net income eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008"
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Per State Plan Exhibits 1 & 2:
        # Applicants: $90 + childcare, compare to Payment Standard
        # Recipients: $90 + childcare + $30+1/3, compare to Standard of Need
        is_enrolled = spm_unit("is_tanf_enrolled", period)

        # Applicant net income: $90 + childcare only
        applicant_net_income = spm_unit("de_tanf_net_income", period)
        payment_standard = spm_unit("de_tanf_payment_standard", period)
        applicant_eligible = applicant_net_income <= payment_standard

        # Recipient countable income: $90 + $30 + 1/3 + childcare
        countable_income = spm_unit("de_tanf_countable_income", period)
        standard_of_need = spm_unit("de_tanf_standard_of_need", period)
        recipient_eligible = countable_income <= standard_of_need

        return where(is_enrolled, recipient_eligible, applicant_eligible)



from policyengine_us.model_api import *


class ok_tanf_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oklahoma TANF payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-59"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ok.dhs.tanf.benefit
        unit_size = spm_unit("spm_unit_size", period)
        return p.payment_standard.calc(unit_size)



from policyengine_us.model_api import *


class mn_k12_education_credit(Variable):
    value_type = float
    entity = TaxUnit
    label = "Minnesota K-12 Education Credit"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/290.0674",
        "https://www.revenue.state.mn.us/sites/default/files/2025-12/m1ed-25.pdf",
    )
    defined_for = "mn_k12_education_credit_eligible"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.states.mn.tax.income.credits.k12_education
        # Count qualifying K-12 children
        k12_children = tax_unit("mn_k12_qualifying_children", period)
        # Get qualifying expenses
        expenses = tax_unit("k12_tuition_and_fees", period)
        # Line 11: expenses * 75%
        expense_credit = expenses * p.expense_rate
        # Line 12: Calculate credit limit based on income
        base_credit_limit = k12_children * p.amount_per_child
        # Phase-out if AGI exceeds threshold
        agi = tax_unit("adjusted_gross_income", period)
        excess_income = max_(0, agi - p.income_threshold)
        # Use different phase-out rate for 1 child vs multiple
        phaseout_rate = where(
            k12_children == 1,
            p.phaseout_rate_one_child,
            p.phaseout_rate_multiple_children,
        )
        phaseout_amount = excess_income * phaseout_rate
        credit_limit = max_(0, base_credit_limit - phaseout_amount)
        # Credit is lesser of expense credit or credit limit
        return min_(expense_credit, credit_limit)



from policyengine_us.model_api import *


class wy_power_shelter_qualified(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Wyoming POWER shelter qualified"
    definition_period = MONTH
    reference = "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/"
    defined_for = StateCode.WY
    default_value = True
    # Shelter qualified (Code N) applies when the unit is responsible for
    # paying all or a portion of shelter costs.
    #
    # Set to False (shelter disqualified) when:
    # - Code Y: No obligation to pay shelter costs (completely furnished)
    # - Code R: Unit lives in government housing subsidy
    # - Code M: Minor parent with dependent children living with parent(s),
    #           adult relative, or court-appointed guardian/custodian
    # - Code S: Unit excludes an individual due to receiving SSI
    #
    # Per ARW Chapter 1, Section 9.



from policyengine_us.model_api import *


class va_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "VA TANF countable income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.VA
    reference = "https://www.dss.virginia.gov/files/division/bp/tanf/manual/300_11-20.pdf#page=50"

    adds = [
        "va_tanf_countable_earned_income",
        "va_tanf_countable_unearned_income",
    ]



from policyengine_us.model_api import *


class nh_fanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Hampshire FANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.dhhs.nh.gov/famar_htm/html/603_01_earned_income_disregards_eid_sr_97-03_02_97_fam_a.htm",
        "https://www.dhhs.nh.gov/sr_htm/html/sr_97-03_dated_02_97.htm",
        "https://www.dhhs.nh.gov/sites/g/files/ehbemt476/files/documents2/tanf-state-plan.pdf#page=29",
    )
    defined_for = StateCode.NH

    def formula(spm_unit, period, parameters):
        # NH has different disregard rates for applicants (20%) vs recipients
        # Recipients: 50% (1997) → 75% (Sept 2022 Cliff Effect Initiative)
        # Note: A 100% disregard exists for high-demand jobs (per NH Workforce
        # Innovation Opportunities Act board) with medical benefits offered,
        # but this is not implemented due to lack of job classification data.
        p = parameters(
            period
        ).gov.states.nh.dhhs.fanf.income.earned_income_disregard
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        is_enrolled = spm_unit("is_tanf_enrolled", period)
        disregard_rate = where(is_enrolled, p.recipient_rate, p.applicant_rate)
        after_disregard = gross_earned * (1 - disregard_rate)

        # Child care deduction applies to earned income (FAM 603.05)
        child_care_deduction = spm_unit("nh_fanf_child_care_deduction", period)
        return max_(after_disregard - child_care_deduction, 0)



from policyengine_us.model_api import *


class aca_required_contribution_percentage(Variable):
    value_type = float
    entity = TaxUnit
    label = "ACA PTC phase-out rate (i.e., IRS Form 8962 'applicable figure')"
    unit = "/1"
    definition_period = YEAR
    reference = "https://www.law.cornell.edu/uscode/text/26/36B#b_3_A"

    def formula(tax_unit, period, parameters):
        magi_frac = tax_unit("aca_magi_fraction", period)
        p = parameters(period).gov.aca.required_contribution_percentage

        # Get bracket boundaries and rates
        thresholds = np.array(p.threshold)
        initial_rates = np.array(p.initial)
        final_rates = np.array(p.final)

        # Find which bracket each household falls into
        # searchsorted returns index where magi_frac would be inserted
        # subtract 1 to get the bracket index (clamped to valid range)
        # Note: there are len(thresholds)-1 brackets (rate arrays are 1 shorter than thresholds)
        bracket_idx = clip(
            np.searchsorted(thresholds, magi_frac, side="right") - 1,
            0,
            len(initial_rates) - 1,
        )

        # Get bracket boundaries for interpolation
        bracket_start = thresholds[bracket_idx]
        # For the last bracket, use a dummy end value (won't affect result since initial=final)
        # Need to use min_ to avoid index errors with vectorized operations
        next_idx = min_(bracket_idx + 1, len(thresholds) - 1)
        bracket_end = where(
            bracket_idx < len(thresholds) - 1,
            thresholds[next_idx],
            bracket_start + 1,  # Dummy value for last bracket
        )

        # Calculate position within bracket (0 to 1)
        bracket_width = bracket_end - bracket_start
        position = where(
            bracket_width > 0,
            (magi_frac - bracket_start) / bracket_width,
            0,
        )
        position = clip(position, 0, 1)

        # Interpolate between initial and final rates
        initial = initial_rates[bracket_idx]
        final = final_rates[bracket_idx]
        return initial + position * (final - initial)



from policyengine_us.model_api import *


class ut_fep_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Utah TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        # Utah earned income after deductions per R986-200-239:
        # 1. Work expense allowance: $100 per employed person
        # 2. 50% earned income disregard on remaining earned income
        p = parameters(period).gov.states.ut.dwf.fep.income.deductions
        after_work_expense = add(
            spm_unit, period, ["ut_fep_earned_income_after_work_expense"]
        )
        # Countable = remainder * (1 - disregard_rate) = remainder * 0.5
        return after_work_expense * (1 - p.earned_income_disregard.rate)



from policyengine_us.model_api import *


class ky_ktap_dependent_care_disregard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kentucky K-TAP dependent care disregard"
    unit = USD
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        # Per 921 KAR 2:016 Section 5(3)(b):
        # Dependent care disregard caps:
        # - $200/month for child under age 2
        # - $175/month for child age 2-12 (full-time)
        # - $150/month for child age 2-12 (part-time)
        # - $0 for age 13+
        # Note: Full-time/part-time distinction (30 hrs/week threshold)
        # is not currently modeled; using full-time rate ($175).
        p = parameters(period).gov.states.ky.dcbs.ktap.income.deductions
        person = spm_unit.members
        age = person("age", period.this_year)
        max_per_child = p.dependent_care.calc(age)
        total_max_disregard = spm_unit.sum(max_per_child)
        childcare_expenses = spm_unit("childcare_expenses", period)
        return min_(childcare_expenses, total_max_disregard)



from policyengine_us.model_api import *


class in_tanf_countable_earned_income_for_eligibility(Variable):
    value_type = float
    entity = SPMUnit
    label = (
        "Indiana TANF countable earned income for eligibility determination"
    )
    unit = USD
    definition_period = MONTH
    reference = (
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-4",
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-2-3",
    )
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        # NOTE: The $30 and 1/3 disregard only applies for the first 4 months
        # of continuing eligibility. This simplified implementation always
        # applies the $30 and 1/3 disregard.
        p = parameters(period).gov.states["in"].fssa.tanf.income.deductions
        person = spm_unit.members
        gross_earned = person("tanf_gross_earned_income", period)

        # $90 per earner, then $30 + 1/3 per unit (470 IAC 10.3-4-4(c))
        after_work_expense = spm_unit.sum(
            max_(gross_earned - p.work_expense.amount, 0)
        )
        after_flat = max_(
            after_work_expense - p.eligibility.flat_disregard.amount, 0
        )
        return after_flat * (1 - p.eligibility.earned_income_disregard.rate)



from policyengine_us.model_api import *


class ca_calworks_stage_3_payment(Variable):
    value_type = float
    entity = Person
    label = "California CalWORKs Stage 3 child care payment"
    unit = USD
    definition_period = MONTH
    defined_for = "ca_calworks_stage_3_eligible"
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?sectionNum=10372.5.&lawCode=WIC"

    def formula(person, period, parameters):
        # Uses same RMR payment structure as Stage 1 and Stage 2
        payment_standard = person(
            "ca_calworks_child_care_payment_standard", period
        )
        time_coefficient = person(
            "ca_calworks_child_care_time_coefficient", period
        )
        payment_factor = person(
            "ca_calworks_child_care_payment_factor", period
        )
        childcare_expenses = person("pre_subsidy_childcare_expenses", period)

        return min_(
            payment_standard * time_coefficient * payment_factor,
            childcare_expenses,
        )



from policyengine_us.model_api import *


class il_hbi_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Illinois Health Benefits for Immigrants"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://hfs.illinois.gov/medicalclients/healthbenefitsforimmigrants.html",
        "https://www.dhs.state.il.us/page.aspx?item=161600",
        "https://hfs.illinois.gov/medicalprograms/allkids/about.html",
    )
    # Illinois Health Benefits for Immigrants (HBI) is a state-funded program
    # providing health coverage to income-eligible residents who are not eligible
    # for federal Medicaid due to their immigration status.
    #
    # The program includes three components:
    # - All Kids: Children 0-18 up to 318% FPL
    # - HBIA (Health Benefits for Immigrant Adults): Ages 42-64 up to 138% FPL
    # - HBIS (Health Benefits for Immigrant Seniors): Ages 65+ up to 100% FPL
    #
    # Note: Ages 19-41 are not covered.

    def formula(person, period, parameters):
        p = parameters(period).gov.states.il.hfs.hbi.eligibility

        # Check base eligibility criteria
        immigration_eligible = person(
            "il_hbi_immigration_status_eligible", period
        )
        age_eligible = person("il_hbi_age_eligible", period)
        income_eligible = person("il_hbi_income_eligible", period)
        resource_eligible = person("il_hbi_resource_eligible", period)

        # Determine age group for program-specific in_effect checks
        age = person("age", period)
        is_child = age <= p.child.max_age
        is_adult = (age >= p.adult.min_age) & (age <= p.adult.max_age)
        is_senior = age >= p.senior.min_age

        # Check if the relevant program is in effect for each age group
        child_in_effect = p.child.in_effect
        adult_in_effect = p.adult.in_effect
        senior_in_effect = p.senior.in_effect

        program_in_effect = (
            (is_child & child_in_effect)
            | (is_adult & adult_in_effect)
            | (is_senior & senior_in_effect)
        )

        return (
            immigration_eligible
            & age_eligible
            & income_eligible
            & resource_eligible
            & program_in_effect
        )



from policyengine_us.model_api import *


class wy_power_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wyoming POWER countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://dfs.wyo.gov/accordions/snap-and-power-policy-manual-1100-extended-menu/",
        "https://wyoleg.gov/statutes/compress/title42.pdf#page=5",
    )
    defined_for = StateCode.WY

    def formula(spm_unit, period, parameters):
        # Per Section 1101: Countable earned = Gross earned - Disregard
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        disregard = spm_unit("wy_power_earned_income_disregard", period)
        return max_(gross_earned - disregard, 0)



from policyengine_us.model_api import *


class months_since_calworks_exit(Variable):
    value_type = int
    entity = SPMUnit
    label = "Months since CalWORKs cash aid ended"
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?sectionNum=10372.&nodeTreePath=16.4.19&lawCode=WIC"



from policyengine_us.model_api import *


class is_qi_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Qualifying Individual (QI) eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/cfr/text/42/435.123",
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
    )

    def formula(person, period, parameters):
        # QI requires income above 120% FPL but at or below 135% FPL
        # QI is only available for people who don't qualify for other Medicaid
        p = parameters(
            period
        ).gov.hhs.medicare.savings_programs.eligibility.income

        medicare_eligible = person("is_medicare_eligible", period.this_year)
        asset_eligible = person("msp_asset_eligible", period)

        fpg = person("msp_fpg", period)
        countable_income = person("msp_countable_income", period)

        slmb_income_limit = fpg * p.slmb.fpl_limit
        qi_income_limit = fpg * p.qi.fpl_limit

        income_above_slmb = countable_income > slmb_income_limit
        income_at_or_below_qi = countable_income <= qi_income_limit
        income_eligible = income_above_slmb & income_at_or_below_qi

        # QI excludes those eligible for other Medicaid coverage
        medicaid_eligible = person("is_medicaid_eligible", period.this_year)

        return (
            medicare_eligible
            & income_eligible
            & asset_eligible
            & ~medicaid_eligible
        )



from policyengine_us.model_api import *


class il_ipass_assist_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    label = "Illinois I-PASS Assist income eligible"
    reference = (
        "https://agency.illinoistollway.com/assist",
        "https://www.dhs.state.il.us/page.aspx?item=150431",
    )
    defined_for = StateCode.IL

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.il.tollway.ipass_assist.eligibility
        fpg = spm_unit("spm_unit_fpg", period)
        # Income verified by Illinois Department of Revenue
        gross_income = add(spm_unit, period, ["irs_gross_income"])
        return gross_income <= fpg * p.fpg_limit



from policyengine_us.model_api import *


class wy_power_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wyoming POWER payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/table-ii-power-income-limits/",
        "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/",
    )
    defined_for = StateCode.WY

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.wy.dfs.power.payment_standard
        unit_size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(unit_size, p.max_unit_size)
        shelter_qualified = spm_unit("wy_power_shelter_qualified", period)
        # Per Section 905: Two payment schedules based on shelter status
        # Per Table II: Sizes 7-12 have same payment standard
        return where(
            shelter_qualified,
            p.shelter_qualified[capped_size],
            p.shelter_disqualified[capped_size],
        )



from policyengine_us.model_api import *


class ny_itemized_deductions_max(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY itemized deductions before reduction"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.nysenate.gov/legislation/laws/TAX/615",
        "https://www.tax.ny.gov/pit/file/itemized-deductions.htm",
    )
    defined_for = StateCode.NY
    documentation = """
    NY Tax Law § 615 requires itemized deductions to be computed using
    federal rules as they existed immediately prior to the enactment of
    Public Law 115-97 (TCJA). This means:
    - No SALT cap ($10,000 limit does not apply)
    - State/local income taxes are NOT deductible (only sales + property)
    - Miscellaneous deductions still allowed with 2% AGI floor
    - Casualty losses not limited to federally declared disasters
    - NY-specific college tuition deduction addition per § 615(d)
    """
    adds = [
        # Federal deductions that don't differ from pre-TCJA
        "charitable_deduction",
        "interest_deduction",
        "medical_expense_deduction",
        # NY-specific pre-TCJA versions
        "ny_salt_deduction",
        "ny_misc_deduction",
        "ny_casualty_loss_deduction",
        # NY-specific addition per Tax Law § 615(d)
        "ny_college_tuition_deduction",
    ]



from policyengine_us.model_api import *


class ms_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Mississippi TANF resources eligibility"
    definition_period = MONTH
    defined_for = StateCode.MS
    reference = (
        "https://www.mdhs.ms.gov/wp-content/uploads/2018/02/MDHS_TANF-Eligibility-Flyer.pdf",
        "https://www.law.cornell.edu/regulations/mississippi/Miss-Code-tit-18-pt-19",
        "https://www.mdhs.ms.gov/help/tanf/",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ms.dhs.tanf.resources
        household_assets = spm_unit("spm_unit_assets", period.this_year)
        return household_assets <= p.limit



from policyengine_us.model_api import *


class ct_tfa_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Connecticut Temporary Family Assistance (TFA)"
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"
    defined_for = StateCode.CT

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_status_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        resources_eligible = spm_unit("ct_tfa_resources_eligible", period)
        income_eligible = spm_unit("ct_tfa_income_eligible", period)
        return (
            demographic_eligible
            & immigration_status_eligible
            & resources_eligible
            & income_eligible
        )



from policyengine_us.model_api import *


class mo_tanf_income_for_need_test(Variable):
    value_type = float
    entity = SPMUnit
    label = "Missouri TANF income for Standard of Need test"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
        "https://dssmanuals.mo.gov/temporary-assistance-case-management/0210-010-10/",
    )
    defined_for = StateCode.MO

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.mo.dss.tanf.earned_income_disregard
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        gross_unearned = add(spm_unit, period, ["tanf_gross_unearned_income"])
        work_expense = min_(gross_earned, p.amount)
        child_care = spm_unit("mo_tanf_child_care_deduction", period)
        countable_earned = max_(gross_earned - work_expense - child_care, 0)
        return countable_earned + gross_unearned



from policyengine_us.model_api import *


class va_tanf_eligibility(Variable):
    value_type = bool
    entity = SPMUnit
    label = "VA TANF eligibility"
    definition_period = MONTH
    defined_for = StateCode.VA
    reference = "https://www.dss.virginia.gov/files/division/bp/tanf/manual/200_7-20.pdf#page=3"

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        income_eligible = spm_unit("va_tanf_income_eligibility", period)
        # At least one person must be a citizen or qualified noncitizen
        has_eligible_member = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        # no resource test for VA TANF
        return demographic_eligible & income_eligible & has_eligible_member



from policyengine_us.model_api import *


class ky_ktap_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kentucky K-TAP countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        # Per 921 KAR 2:016 Section 5(3):
        # (a) Deduct $175 work expense from gross earned income
        # (b) Deduct dependent care expenses up to limits
        # (e) Apply 50% earned income disregard for 6 months
        p = parameters(period).gov.states.ky.dcbs.ktap.income.deductions
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        after_work_expense = max_(gross_earned - p.work_expense, 0)
        dependent_care = spm_unit("ky_ktap_dependent_care_disregard", period)
        after_dependent_care = max_(after_work_expense - dependent_care, 0)
        return after_dependent_care * (1 - p.earned_income_disregard_rate)



from policyengine_us.model_api import *


class me_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Maine TANF income eligible"
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.law.cornell.edu/regulations/maine/10-144-C-M-R-ch-331-app-Charts",
    )
    defined_for = StateCode.ME

    def formula(spm_unit, period, parameters):
        # Per 22 M.R.S. Section 3762(3)(B)(7-F):
        # Countable income must not exceed the Standard of Need
        countable_income = spm_unit("me_tanf_countable_income", period)
        standard_of_need = spm_unit("me_tanf_standard_of_need", period)
        return countable_income <= standard_of_need



from policyengine_us.model_api import *


class ny_misc_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY miscellaneous deduction"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.nysenate.gov/legislation/laws/TAX/615",
        "https://www.tax.ny.gov/pit/file/itemized-deductions.htm",
    )
    defined_for = StateCode.NY
    documentation = """
    NY Tax Law § 615 requires itemized deductions to be computed using
    pre-TCJA federal rules. Unlike federal rules post-2017, NY still
    allows miscellaneous itemized deductions subject to the 2% AGI floor.
    These include unreimbursed employee expenses and tax preparation fees.
    """

    def formula(tax_unit, period, parameters):
        # NY uses pre-TCJA rules: misc deductions are still allowed
        # with the 2% AGI floor (not suspended like federal post-2017)
        p = parameters(period).gov.irs.deductions.itemized.misc
        expenses = tax_unit("total_misc_deductions", period)
        misc_floor = p.floor * tax_unit("positive_agi", period)
        return max_(0, expenses - misc_floor)



from policyengine_us.model_api import *


class mn_mfip(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP"
    unit = USD
    definition_period = MONTH
    reference = "https://www.revisor.mn.gov/statutes/cite/142G.16"
    defined_for = "mn_mfip_eligible"

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.16, Subd. 2 (benefit calculation):
        # (a) FWL - countable earned = X, cap at transitional standard
        # (d) X - unearned = final benefit
        # Note: Dependent care deduction does NOT apply to benefit calculation
        transitional_standard = spm_unit(
            "mn_mfip_transitional_standard", period
        )
        family_wage_level = spm_unit("mn_mfip_family_wage_level", period)
        countable_earned = spm_unit("mn_mfip_countable_earned_income", period)
        countable_unearned = spm_unit(
            "mn_mfip_countable_unearned_income", period
        )
        # Step 1: Subtract countable earned from FWL, floor at 0
        fwl_minus_earned = max_(family_wage_level - countable_earned, 0)
        # Step 2: Cap at transitional standard
        earned_based_benefit = min_(fwl_minus_earned, transitional_standard)
        # Step 3: Subtract countable unearned income, floor at zero
        return max_(earned_based_benefit - countable_unearned, 0)



from policyengine_us.model_api import *


class tn_ff_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Tennessee Families First eligible"
    definition_period = MONTH
    reference = "https://www.tn.gov/humanservices/for-families/families-first-tanf/families-first-eligibility-information.html"
    defined_for = StateCode.TN

    def formula(spm_unit, period, parameters):
        # Must meet demographic requirements (minor child with parent/relative)
        # Use federal demographic eligibility directly
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Must have at least one citizen or legal immigrant
        # Use federal immigration eligibility directly
        has_citizen = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        # Must meet income eligibility
        income_eligible = spm_unit("tn_ff_income_eligible", period)

        # Must meet resource eligibility
        resources_eligible = spm_unit("tn_ff_resources_eligible", period)

        # All requirements must be met
        return (
            demographic_eligible
            & has_citizen
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class hi_tanf_countable_earned_income_person(Variable):
    value_type = float
    entity = Person
    label = "Hawaii TANF countable earned income per person"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=19",
    )
    defined_for = StateCode.HI

    def formula(person, period, parameters):
        p = parameters(period).gov.states.hi.dhs.tanf.deductions

        # Start with person's gross earned income (federal baseline)
        gross_earned = person("tanf_gross_earned_income", period)

        # Step 1: Apply 20% standard deduction
        after_standard = gross_earned * (1 - p.standard_rate)

        # Step 2: Apply $200 flat rate deduction (per person)
        after_flat = max_(after_standard - p.flat_amount, 0)

        # Step 3: Apply 55% earned income disregard
        # NOTE: 55% applies for months 1-24; 36% applies after month 24
        # 55% is used as default here
        return after_flat * (1 - p.eidr_rate)



from policyengine_us.model_api import *


class ok_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Oklahoma TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-59"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        # Per OAC 340:10-3-39: Unearned income counted at gross amount
        income = add(
            spm_unit,
            period,
            ["ok_tanf_countable_earned_income", "tanf_gross_unearned_income"],
        )
        # Per OAC 340:10-3-33(b): Deduct dependent care expenses
        dependent_care_deduction = spm_unit(
            "ok_tanf_dependent_care_deduction", period
        )
        return max_(income - dependent_care_deduction, 0)



from policyengine_us.model_api import *


class mn_mfip_child_support_income_exclusion(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP child support income exclusion"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/256P.06#stat.256P.06.3"
    )
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 256P.06, Subd. 3:
        # Child support up to $100 (1 child) or $200 (2+ children) is excluded.
        p = parameters(
            period
        ).gov.states.mn.dcyf.mfip.income.deductions.child_support
        child_support = add(spm_unit, period, ["child_support_received"])
        children = spm_unit("spm_unit_count_children", period.this_year)
        max_disregard = p.calc(children)
        return min_(child_support, max_disregard)



from policyengine_us.model_api import *


class in_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Indiana Temporary Assistance for Needy Families (TANF)"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-2-5",
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-4",
    )
    defined_for = "in_tanf_eligible"

    def formula(spm_unit, period, parameters):
        payment_standard = spm_unit("in_tanf_payment_standard", period)
        countable_income = spm_unit(
            "in_tanf_countable_income_for_payment", period
        )
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class me_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Maine TANF"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.law.cornell.edu/regulations/maine/10-144-C-M-R-ch-331-app-Charts",
    )
    defined_for = "me_tanf_eligible"

    def formula(spm_unit, period, parameters):
        # Per 22 M.R.S. Section 3762(3)(B)(8):
        # Benefit = min(Maximum Payment Level, Standard of Need - Countable Income)
        standard_of_need = spm_unit("me_tanf_standard_of_need", period)
        countable_income = spm_unit("me_tanf_countable_income", period)
        maximum_benefit = spm_unit("me_tanf_maximum_benefit", period)

        computed_benefit = max_(standard_of_need - countable_income, 0)
        # NOTE: Maine also provides a Special Need Housing Allowance (SNHA) of up
        # to $300/month for households with housing costs >= 50% of countable
        # income. SNHA is not modeled. See 10-144 C.M.R. Chapter 331, Section G(4).
        return min_(computed_benefit, maximum_benefit)



from policyengine_us.model_api import *


class has_no_prior_formal_early_learning(Variable):
    value_type = bool
    entity = Person
    label = "Has not previously participated in formal early learning"
    definition_period = YEAR
    documentation = "Whether the child has NOT previously participated in a formal early learning program (e.g., Head Start, preschool, licensed childcare)"



from policyengine_us.model_api import *


class ut_fep_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Utah Family Employment Program due to resources"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-230"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ut.dwf.fep.resources.limit
        # Use federal spm_unit_assets variable directly
        assets = spm_unit("spm_unit_assets", period.this_year)
        return assets <= p.amount



from policyengine_us.model_api import *


class hi_tanf_dependent_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Hawaii TANF dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=20",
    )
    defined_for = StateCode.HI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.hi.dhs.tanf.deductions.dependent_care

        # Get actual childcare expenses (YEAR variable, auto-converts to monthly)
        expenses = spm_unit("childcare_expenses", period)

        # Count children in the unit for the per-child cap
        person = spm_unit.members
        is_child = person("is_child", period.this_year)
        num_children = spm_unit.sum(is_child)

        # Determine rate based on MAX of adult hours
        # If either parent works full-time, use full-time rate
        is_adult = ~person("is_child", period.this_year)
        hours = person("weekly_hours_worked_before_lsr", period.this_year)
        # Only consider adult hours (children get 0)
        adult_hours = where(is_adult, hours, 0)
        max_adult_hours = spm_unit.max(adult_hours)

        # Full-time = 32+ hours/week
        # Rate is $175 (full-time) or $165 (part-time) per child
        rate = p.amount.calc(max_adult_hours)
        max_deduction = num_children * rate

        # Deduct actual cost up to max per child
        return min_(expenses, max_deduction)



from policyengine_us.model_api import *


class de_tanf_countable_earned_income_person(Variable):
    value_type = float
    entity = Person
    label = "Delaware TANF countable earned income per person"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://dhss.delaware.gov/wp-content/uploads/sites/11/dss/pdf/detanfstateplan2017.pdf#page=6",
    )
    defined_for = StateCode.DE

    def formula(person, period, parameters):
        # Per DSSM 4008 / State Plan Exhibit 1 Step 3:
        # For benefit calculation, subtract from each earner's income:
        # 1. $90 standard work expense
        # 2. $30 flat disregard
        # 3. 1/3 of remainder
        #
        # NOTE: The $30+1/3 disregard is limited to 4 consecutive months,
        # followed by $30 only for 8 months, then neither until 12 months
        # without TANF. We always apply the full $30+1/3 disregard.
        # See: https://help.workworldapp.com/wwwebhelp/de_earned_income_disregards_tanf_and_ga.htm
        p = parameters(period).gov.states.de.dhss.tanf.income

        gross_earned = person("tanf_gross_earned_income", period)
        has_earnings = gross_earned > 0

        # Step 1: Subtract $90 work expense per earner
        after_work_expense = max_(
            gross_earned - p.deductions.work_expense * has_earnings, 0
        )

        # Step 2: Subtract $30 flat disregard per earner
        after_thirty = max_(
            after_work_expense
            - p.deductions.earned_income_disregard.flat * has_earnings,
            0,
        )

        # Step 3: Subtract 1/3 of remainder
        one_third_disregard = (
            after_thirty * p.deductions.earned_income_disregard.percentage
        )
        return max_(after_thirty - one_third_disregard, 0)



from policyengine_us.model_api import *


class il_pfae_is_deep_poverty(Variable):
    value_type = bool
    entity = Person
    label = "Family income at or below 50% FPL (50-point factor for IL PFAE)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/pdg-eg-grant-enrollment-form.pdf#page=2",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Deep poverty (income <= 50% FPL) is a highest priority factor.
        # Per the ISBE PDG enrollment form, Factor 4 includes "Family income at
        # or below 50% of the federal poverty guidelines."
        p = parameters(period).gov.states.il.isbe.pfae.eligibility.income
        spm_unit = person.spm_unit
        fpg = spm_unit("spm_unit_fpg", period)
        income = spm_unit("il_isbe_countable_income", period)
        threshold = fpg * p.deep_poverty_rate
        return income <= threshold



from policyengine_us.model_api import *


class ne_adc_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nebraska ADC countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1726",
        "https://nebraskalegislature.gov/laws/statutes.php?statute=68-1713",
    )
    defined_for = StateCode.NE
    adds = [
        "ne_adc_countable_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class de_tanf_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Delaware TANF payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008",
        "https://dhss.delaware.gov/dss/tanf/",
    )
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.de.dhss.tanf.payment_standard
        unit_size = spm_unit("spm_unit_size", period)
        capped_unit_size = min_(unit_size, p.max_unit_size)
        base_amount = p.amount[capped_unit_size]
        # For each additional person beyond max_unit_size, add $69
        additional_persons = max_(unit_size - p.max_unit_size, 0)
        additional_amount = additional_persons * p.additional_person
        return base_amount + additional_amount



from policyengine_us.model_api import *


class id_tafi_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Idaho TAFI countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.252"
    )
    defined_for = StateCode.ID

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.id.tafi.income.earned
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        # Per IDAPA 16.03.08.252: 60% of gross earned income is subtracted
        # from the work incentive table amount
        return gross_earned * p.countable_rate



from policyengine_us.model_api import *


class ky_ktap_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kentucky K-TAP countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        return add(
            spm_unit,
            period,
            [
                "ky_ktap_countable_earned_income",
                "ky_ktap_countable_unearned_income",
            ],
        )



from policyengine_us.model_api import *


class ok_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Oklahoma TANF"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/title-340/chapter-10"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        # Use federal demographic eligibility (minor child or pregnant)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        # Per OAC 340:10-15: Must be US citizen or qualified alien
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        income_eligible = spm_unit("ok_tanf_income_eligible", period)
        resources_eligible = spm_unit("ok_tanf_resources_eligible", period)

        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class ri_works_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Rhode Island Works countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/rhode-island/218-RICR-20-00-2.15",
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
    )
    defined_for = StateCode.RI

    def formula(spm_unit, period, parameters):
        # Per 218-RICR-20-00-2.15: $525 + 50% disregard applies per earner
        earned_after_disregard = add(
            spm_unit, period, ["ri_works_earned_income_after_disregard_person"]
        )
        # Per 218-RICR-20-00-2.15.5(B)(2): Dependent care deduction
        dependent_care = spm_unit("ri_works_dependent_care_deduction", period)
        return max_(earned_after_disregard - dependent_care, 0)



from policyengine_us.model_api import *


class va_up_tanf_eligibility(Variable):
    value_type = bool
    entity = SPMUnit
    label = "VA TANF-UP eligibility"
    definition_period = MONTH
    defined_for = StateCode.VA
    reference = "https://www.dss.virginia.gov/files/division/bp/tanf/manual/700_07-20.pdf#page=2"

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        head_or_spouse = person("is_tax_unit_head_or_spouse", period.this_year)
        disabled = person("is_disabled", period.this_year)
        able_bodied_adult = head_or_spouse & ~disabled
        return spm_unit.sum(able_bodied_adult) > 1



from policyengine_us.model_api import *


class il_isbe_income_eligible(Variable):
    value_type = bool
    entity = Person
    label = (
        "Meets income requirements for Illinois ISBE early childhood programs"
    )
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/pages/preschool-for-all.aspx",
        "https://law.onecle.com/illinois/105ilcs5/2-3.71.html",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Income must be at or below 400% FPL for program eligibility.
        # 105 ILCS 5/2-3.71 establishes "4 times the poverty guidelines" as the
        # maximum income threshold for preschool program eligibility.
        p = parameters(period).gov.states.il.isbe.income
        spm_unit = person.spm_unit
        fpg = spm_unit("spm_unit_fpg", period)
        income = spm_unit("il_isbe_countable_income", period)
        threshold = fpg * p.income_limit_rate
        return income <= threshold



from policyengine_us.model_api import *


class dc_ccsp_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "DC Child Care Subsidy Program (CCSP) countable income"
    definition_period = MONTH
    reference = "https://osse.dc.gov/sites/default/files/dc/sites/osse/publication/attachments/DC%20Child%20Care%20Subsidy%20Program%20Policy%20Manual.pdf#page=12"
    defined_for = StateCode.DC

    adds = "gov.states.dc.dhs.ccsp.income.countable_income.sources"



from policyengine_us.model_api import *


class ny_salt_deduction(Variable):
    value_type = float
    entity = TaxUnit
    label = "NY SALT deduction"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://www.nysenate.gov/legislation/laws/TAX/615",
        "https://www.tax.ny.gov/pit/file/itemized-deductions.htm",
    )
    defined_for = StateCode.NY
    documentation = """
    NY Tax Law § 615 requires itemized deductions to be computed using
    pre-TCJA federal rules. For SALT:
    - No $10,000 cap applies
    - State and local income taxes are NOT deductible (add-back required per § 615(c))
    - State and local sales taxes ARE deductible
    - Real estate taxes ARE deductible
    """

    def formula(tax_unit, period, parameters):
        # NY allows sales tax and real estate taxes, but NOT income taxes
        # Per Tax Law § 615(c), state/local income taxes must be subtracted
        sales_tax = add(
            tax_unit, period, ["state_sales_tax", "local_sales_tax"]
        )
        real_estate_taxes = add(tax_unit, period, ["real_estate_taxes"])
        return sales_tax + real_estate_taxes



from policyengine_us.model_api import *


class wy_power_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Wyoming POWER income eligible"
    definition_period = MONTH
    reference = (
        "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/table-ii-power-income-limits/",
        "https://dfs.wyo.gov/accordions/snap-and-power-policy-manual-1100-extended-menu/",
    )
    defined_for = StateCode.WY

    def formula(spm_unit, period, parameters):
        # Income eligible if countable income < payment standard
        # (i.e., household would receive a positive benefit)
        # Note: The "Maximum Earned Income Limit" in Table II equals
        # payment_standard + earned_income_disregard, which is the
        # break-even point for earned-income-only households.
        countable_income = spm_unit("wy_power_countable_income", period)
        payment_standard = spm_unit("wy_power_payment_standard", period)
        return countable_income < payment_standard



from policyengine_us.model_api import *


class sd_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "South Dakota TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:05:02"
    defined_for = StateCode.SD

    adds = [
        "sd_tanf_countable_earned_income",
        "sd_tanf_countable_unearned_income",
    ]



from policyengine_us.model_api import *


class mo_capital_gains_subtraction_person(Variable):
    value_type = float
    entity = Person
    label = "Missouri capital gains subtraction for each person"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://dor.mo.gov/faq/taxation/individual/capital-gains-subtraction.html",
    )
    defined_for = StateCode.MO

    def formula(person, period, parameters):
        tax_unit = person.tax_unit
        # Get the tax unit level capital gains subtraction
        tax_unit_subtraction = tax_unit("mo_capital_gains_subtraction", period)
        # Allocate proportionally based on each person's share of
        # capital gains (both short-term and long-term) to match
        # the net_capital_gain used at the tax unit level
        person_cg = person("capital_gains", period)
        tax_unit_cg = tax_unit.sum(person_cg)
        # Use mask to avoid divide-by-zero, default to zero allocation
        person_share = np.zeros_like(tax_unit_cg)
        mask = tax_unit_cg > 0
        person_share[mask] = person_cg[mask] / tax_unit_cg[mask]
        return person_share * tax_unit_subtraction



from policyengine_us.model_api import *
from policyengine_core.periods import period as period_


def create_sc_h3492_eitc_refundable() -> Reform:
    """
    South Carolina H.3492 - Partially Refundable EITC

    Amends SC Code Section 12-6-3632 to make a portion of the state EITC
    refundable. If the SC EITC (125% of federal) exceeds the taxpayer's
    state income tax liability, 25% of the excess is refunded.

    Reference: https://www.scstatehouse.gov/sess126_2025-2026/bills/3492.htm
    """

    class sc_h3492_total_eitc(Variable):
        value_type = float
        entity = TaxUnit
        label = "SC H.3492 total EITC amount"
        unit = USD
        definition_period = YEAR
        reference = (
            "https://www.scstatehouse.gov/sess126_2025-2026/bills/3492.htm",
            "https://www.scstatehouse.gov/code/t12c006.php",
        )
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            federal_eitc = tax_unit("eitc", period)
            rate = parameters(
                period
            ).gov.states.sc.tax.income.credits.eitc.rate
            return np.round(federal_eitc * rate, 1)

    class sc_h3492_other_non_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        label = "SC non-refundable credits excluding EITC"
        unit = USD
        definition_period = YEAR
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            # Sum non-refundable credits excluding EITC
            cdcc = tax_unit("sc_cdcc", period)
            two_wage = tax_unit("sc_two_wage_earner_credit", period)
            return cdcc + two_wage

    class sc_h3492_tax_liability_for_eitc(Variable):
        value_type = float
        entity = TaxUnit
        label = "SC tax liability available for EITC"
        unit = USD
        definition_period = YEAR
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            # Tax before non-refundable credits minus other credits
            tax_before = tax_unit(
                "sc_income_tax_before_non_refundable_credits", period
            )
            other_credits = tax_unit(
                "sc_h3492_other_non_refundable_credits", period
            )
            return max_(0, tax_before - other_credits)

    class sc_h3492_eitc_non_refundable(Variable):
        value_type = float
        entity = TaxUnit
        label = "SC H.3492 EITC non-refundable portion"
        unit = USD
        definition_period = YEAR
        reference = (
            "https://www.scstatehouse.gov/sess126_2025-2026/bills/3492.htm",
        )
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            total_eitc = tax_unit("sc_h3492_total_eitc", period)
            tax_liability = tax_unit("sc_h3492_tax_liability_for_eitc", period)
            # Non-refundable portion is limited to tax liability
            return min_(total_eitc, tax_liability)

    class sc_h3492_eitc_refundable(Variable):
        value_type = float
        entity = TaxUnit
        label = "SC H.3492 EITC refundable portion"
        unit = USD
        definition_period = YEAR
        reference = (
            "https://www.scstatehouse.gov/sess126_2025-2026/bills/3492.htm",
        )
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.contrib.states.sc.h3492
            total_eitc = tax_unit("sc_h3492_total_eitc", period)
            non_refundable = tax_unit("sc_h3492_eitc_non_refundable", period)
            excess = max_(0, total_eitc - non_refundable)
            return excess * p.refundable_excess_rate

    class sc_non_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        label = "South Carolina non-refundable credits"
        unit = USD
        definition_period = YEAR
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            # Other non-refundable credits plus the non-refundable EITC portion
            other_credits = tax_unit(
                "sc_h3492_other_non_refundable_credits", period
            )
            eitc_non_refundable = tax_unit(
                "sc_h3492_eitc_non_refundable", period
            )
            return other_credits + eitc_non_refundable

    class sc_refundable_credits(Variable):
        value_type = float
        entity = TaxUnit
        label = "South Carolina refundable credits"
        unit = USD
        definition_period = YEAR
        defined_for = StateCode.SC

        def formula(tax_unit, period, parameters):
            p = parameters(period).gov.states.sc.tax.income.credits
            standard_credits = add(tax_unit, period, p.refundable)
            # Add refundable EITC portion from H.3492
            eitc_refundable = tax_unit("sc_h3492_eitc_refundable", period)
            return standard_credits + eitc_refundable

    class reform(Reform):
        def apply(self):
            self.update_variable(sc_h3492_total_eitc)
            self.update_variable(sc_h3492_other_non_refundable_credits)
            self.update_variable(sc_h3492_tax_liability_for_eitc)
            self.update_variable(sc_h3492_eitc_non_refundable)
            self.update_variable(sc_h3492_eitc_refundable)
            self.update_variable(sc_non_refundable_credits)
            self.update_variable(sc_refundable_credits)

    return reform


def create_sc_h3492_eitc_refundable_reform(
    parameters, period, bypass: bool = False
):
    if bypass:
        return create_sc_h3492_eitc_refundable()

    p = parameters.gov.contrib.states.sc.h3492

    reform_active = False
    current_period = period_(period)

    for i in range(5):
        if p(current_period).in_effect:
            reform_active = True
            break
        current_period = current_period.offset(1, "year")

    if reform_active:
        return create_sc_h3492_eitc_refundable()
    else:
        return None


sc_h3492_eitc_refundable = create_sc_h3492_eitc_refundable_reform(
    None, None, bypass=True
)



from policyengine_us.model_api import *


class nd_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for North Dakota TANF"
    definition_period = MONTH
    reference = "https://www.nd.gov/dhs/policymanuals/40019/400_19_110_15.htm"
    defined_for = StateCode.ND

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        income_eligible = spm_unit("nd_tanf_income_eligible", period)
        resources_eligible = spm_unit("nd_tanf_resources_eligible", period)
        return (
            demographic_eligible
            & immigration_eligible
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class nm_works_earned_income_deduction_person(Variable):
    value_type = float
    entity = Person
    label = "New Mexico Works earned income deduction (person)"
    unit = USD
    definition_period = MONTH
    reference = "https://www.srca.nm.gov/parts/title08/08.102.0520.html"
    defined_for = StateCode.NM

    def formula(person, period, parameters):
        # Per 8.102.520.12(C) NMAC:
        # Flat amount ($125 single-parent, $225 two-parent) + 50% of remainder
        p = parameters(
            period
        ).gov.states.nm.hca.nm_works.income.deductions.work_incentive

        gross_earned = person("tanf_gross_earned_income", period)

        # Count adults in the SPM unit to determine single vs two-parent
        spm_unit = person.spm_unit
        adult_count = add(spm_unit, period, ["is_tax_unit_head_or_spouse"])

        # Flat deduction depends on single vs two-parent
        single_parent_flat = p.single_parent.amount
        two_parent_flat = p.two_parent.amount
        disregard_rate = p.rate

        # Head/spouse get $225 in two-parent families, $125 in single-parent
        is_two_parent = adult_count > 1
        flat_amount = where(is_two_parent, two_parent_flat, single_parent_flat)

        # Step 1: Subtract flat amount (not to exceed earnings)
        flat_deduction = min_(flat_amount, gross_earned)
        after_flat = max_(gross_earned - flat_deduction, 0)

        # Step 2: 50% disregard on remainder
        percentage_deduction = after_flat * disregard_rate

        return flat_deduction + percentage_deduction



from policyengine_us.model_api import *


class de_tanf_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Delaware TANF gross income eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008"
    defined_for = StateCode.DE

    def formula(spm_unit, period, parameters):
        # Per DSSM 4008: Gross income <= 185% of Standard of Need
        standard_of_need = spm_unit("de_tanf_standard_of_need", period)
        p = parameters(period).gov.states.de.dhss.tanf.income
        gross_income_limit = standard_of_need * p.gross_limit

        gross_income = spm_unit("de_tanf_gross_income", period)

        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class taxable_social_security_tier_1(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Taxable Social Security (tier 1)"
    documentation = "Taxable Social Security from 0-50% taxation tier, credited to OASDI trust funds"
    unit = USD
    reference = "https://www.law.cornell.edu/uscode/text/26/86#a_1"

    def formula(tax_unit, period, parameters):
        p = parameters(period).gov.irs.social_security.taxability
        gross_ss = tax_unit("tax_unit_social_security", period)
        combined_income = tax_unit(
            "tax_unit_combined_income_for_social_security_taxability", period
        )
        filing_status = tax_unit("filing_status", period)
        status = filing_status.possible_values
        separate = filing_status == status.SEPARATE
        cohabitating = tax_unit("cohabitating_spouses", period)

        base_amount = where(
            separate & cohabitating,
            p.threshold.base.separate_cohabitating,
            p.threshold.base.main[filing_status],
        )
        adjusted_base_amount = where(
            separate & cohabitating,
            p.threshold.adjusted_base.separate_cohabitating,
            p.threshold.adjusted_base.main[filing_status],
        )

        under_first_threshold = combined_income < base_amount
        under_second_threshold = combined_income < adjusted_base_amount

        combined_income_excess = tax_unit(
            "tax_unit_ss_combined_income_excess", period
        )

        # Tier 1 amount (IRC §86(a)(1))
        amount_under_paragraph_1 = min_(
            p.rate.base.benefit_cap * gross_ss,
            p.rate.base.excess * combined_income_excess,
        )

        # Bracket amount when in tier 2 (IRC §86(a)(2)(A)(ii))
        bracket_amount = min_(
            amount_under_paragraph_1,
            p.rate.additional.bracket * (adjusted_base_amount - base_amount),
        )

        return select(
            [under_first_threshold, under_second_threshold],
            [0, amount_under_paragraph_1],
            default=bracket_amount,
        )



from policyengine_us.model_api import *


class id_tafi_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Idaho TAFI eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.100",
        "https://www.law.cornell.edu/regulations/idaho/IDAPA-16.03.08.254",
    )
    defined_for = StateCode.ID

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.id.tafi
        # Demographic eligibility - use federal baseline (IDAPA 16.03.08.125)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Immigration eligibility - use federal baseline (IDAPA 16.03.08.131)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )

        # Resources eligibility (IDAPA 16.03.08.200)
        resources_eligible = spm_unit("id_tafi_resources_eligible", period)

        # Income eligibility - grant calculation must result in positive amount
        grant_standard = spm_unit("id_tafi_grant_standard", period)
        income_eligible = grant_standard > 0

        return (
            demographic_eligible
            & immigration_eligible
            & resources_eligible
            & income_eligible
        )



from policyengine_us.model_api import *


class mi_fip_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Michigan FIP payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/RF/Public/RFT/210.pdf",
        "https://www.michigan.gov/mdhhs/-/media/Project/Websites/mdhhs/Inside-MDHHS/Reports-and-Statistics---Human-Services/State-Plans-and-Federal-Regulations/TANF_State_Plan_2023.pdf",
    )
    defined_for = StateCode.MI

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.mi.mdhhs.fip.payment_standard

        # Determine household size
        size = spm_unit("spm_unit_size", period)

        # For sizes 1-7, use the bracket schedule
        # For size 8+, use the size 7 amount plus additional per person
        max_household_size = p.max_household_size
        capped_size = min_(size, max_household_size)
        base_standard = p.base.calc(capped_size)

        # Add additional amount for household size 8+
        additional_persons = max_(size - max_household_size, 0)
        additional_amount = additional_persons * p.additional

        return base_standard + additional_amount



from policyengine_us.model_api import *


class il_ipass_assist_categorical_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    definition_period = YEAR
    label = "Illinois I-PASS Assist categorically eligible"
    reference = (
        "https://www.dhs.state.il.us/page.aspx?item=150431",
        "https://agency.illinoistollway.com/documents/20184/1344218/2023-06_IPASS%20Assist%20One%20Pager.pdf/11051bbf-1809-6d1b-de08-90ba3d931b23#page=1",
    )
    defined_for = StateCode.IL

    def formula(spm_unit, period, parameters):
        # IL Tollway: eligible if "currently participating in SNAP or TANF"
        snap = spm_unit("snap", period) > 0
        tanf = spm_unit("il_tanf", period) > 0
        return snap | tanf



from policyengine_us.model_api import *


class ms_tanf_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Mississippi TANF maximum benefit"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.MS
    reference = (
        "https://www.mdhs.ms.gov/wp-content/uploads/2018/02/MDHS_TANF-Eligibility-Flyer.pdf",
        "https://billstatus.ls.state.ms.us/documents/2021/html/SB/2700-2799/SB2759SG.htm",
        "https://www.law.cornell.edu/regulations/mississippi/Miss-Code-tit-18-pt-19",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ms.dhs.tanf.payment_standard
        size = spm_unit("spm_unit_size", period.this_year)

        # Payment standard: first_person + second_person (if size >= 2) + additional_person * (size - 2)
        first_person = p.first_person
        second_person = where(size >= 2, p.second_person, 0)
        additional_persons = max_(size - 2, 0) * p.additional_person

        return first_person + second_person + additional_persons



from policyengine_us.model_api import *


class nh_fanf_child_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Hampshire FANF child/dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = "https://www.dhhs.nh.gov/famar_htm/index.htm#html/603_05_child_dependent_care_deduction_sr_12-04_07_12_fam_a.htm"
    defined_for = StateCode.NH

    def formula(spm_unit, period, parameters):
        p = parameters(
            period
        ).gov.states.nh.dhhs.fanf.income.child_care_deduction
        person = spm_unit.members

        # Determine if any earner is full-time (≥$377/month)
        earned_income = person("tanf_gross_earned_income", period)
        is_full_time = earned_income >= p.full_time_threshold
        any_full_time = spm_unit.any(is_full_time)

        # Get child age and status
        age = person("age", period.this_year)
        is_child = person("is_child", period)

        # Calculate max deduction per child based on employment status and age
        full_time_max = p.full_time.calc(age)
        part_time_max = p.part_time.calc(age)
        max_per_child = where(any_full_time, full_time_max, part_time_max)

        # Only count children
        max_deduction_per_child = max_per_child * is_child
        total_max_deduction = spm_unit.sum(max_deduction_per_child)

        # Cap at actual childcare expenses
        childcare_expenses = spm_unit("childcare_expenses", period)

        return min_(childcare_expenses, total_max_deduction)



from policyengine_us.model_api import *


class il_hbi_income_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Meets Illinois Health Benefits for Immigrants income eligibility"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://hfs.illinois.gov/medicalclients/healthbenefitsforimmigrants.html",
        "https://www.dhs.state.il.us/page.aspx?item=161600",
        "https://hfs.illinois.gov/medicalprograms/allkids/about.html",
    )
    # Illinois HBI has different income limits and methodologies by age group:
    # - Children (0-18): Up to 318% FPL (All Kids, uses MAGI)
    # - Adults (42-64): Up to 138% FPL (HBIA, uses MAGI)
    # - Seniors (65+): Up to 100% FPL (HBIS, uses AABD methodology)
    #
    # Per Illinois DHS, HBIS follows AABD community eligibility criteria
    # for income counting, which differs from MAGI.

    def formula(person, period, parameters):
        p = parameters(period).gov.states.il.hfs.hbi.eligibility

        age = person("age", period)

        # Age thresholds
        is_child = age <= p.child.max_age
        is_adult = (age >= p.adult.min_age) & (age <= p.adult.max_age)
        is_senior = age >= p.senior.min_age

        # Get appropriate income level based on age group
        # Children and adults use MAGI, seniors use AABD methodology
        magi_income_level = person("medicaid_income_level", period)
        aabd_income_level = person("il_hbi_senior_income_level", period)

        income_level = where(is_senior, aabd_income_level, magi_income_level)

        # Select appropriate income limit based on age
        income_limit = select(
            [is_child, is_adult, is_senior],
            [
                p.child.income_limit,
                p.adult.income_limit,
                p.senior.income_limit,
            ],
            default=0,  # Ages 19-41 not covered
        )

        return income_level <= income_limit



from policyengine_us.model_api import *


class MSPCategory(Enum):
    NONE = "None"
    QMB = "Qualified Medicare Beneficiary"
    SLMB = "Specified Low-Income Medicare Beneficiary"
    QI = "Qualifying Individual"


class msp_category(Variable):
    value_type = Enum
    possible_values = MSPCategory
    default_value = MSPCategory.NONE
    entity = Person
    label = "Medicare Savings Program category"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
        "https://www.law.cornell.edu/cfr/text/42/435.121",
    )

    def formula(person, period, parameters):
        qmb = person("is_qmb_eligible", period)
        slmb = person("is_slmb_eligible", period)
        qi = person("is_qi_eligible", period)

        # Priority: QMB > SLMB > QI (QMB is most comprehensive)
        return select(
            [qmb, slmb, qi],
            [MSPCategory.QMB, MSPCategory.SLMB, MSPCategory.QI],
            default=MSPCategory.NONE,
        )



from policyengine_us.model_api import *


class nd_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "North Dakota TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.nd.gov/dhs/policymanuals/40019/400_19_110_20.htm"
    defined_for = StateCode.ND

    adds = [
        "nd_tanf_countable_earned_income",
        "nd_tanf_countable_unearned_income",
    ]



from policyengine_us.model_api import *
from policyengine_us.variables.gov.hhs.medicare.savings_programs.category.msp_category import (
    MSPCategory,
)


class msp_benefit_value(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Medicare Savings Program estimated monthly benefit value"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
    )
    defined_for = "msp_eligible"

    def formula(person, period, parameters):
        category = person("msp_category", period)

        # Get premiums (automatically disaggregated from annual to monthly)
        part_a_premium = person("base_part_a_premium", period)
        part_b_premium = person("base_part_b_premium", period)

        # Benefit depends on category:
        # QMB: Part A + Part B premiums (plus deductibles/copays, not modeled)
        # SLMB: Part B premium only
        # QI: Part B premium only
        is_qmb = category == MSPCategory.QMB
        is_slmb = category == MSPCategory.SLMB
        is_qi = category == MSPCategory.QI

        return (
            is_qmb * (part_a_premium + part_b_premium)
            + is_slmb * part_b_premium
            + is_qi * part_b_premium
        )



from policyengine_us.model_api import *


class il_pi_lower_priority_score(Variable):
    value_type = int
    entity = Person
    label = "Illinois PI lower priority score (10 pts factors)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=3",
    )
    defined_for = "il_pi_demographic_eligible"

    def formula(person, period, parameters):
        household = person.household
        spm_unit = person.spm_unit

        # Lower priority factors (10 pts each):
        # Factor 33: Teen parent at birth of first child (SPMUnit-level variable)
        # Parent was under 20 when first child was born
        is_teen_parent_family = spm_unit(
            "il_isbe_was_teen_parent_at_first_birth", period
        )
        # Factor 35: Parent/Caregiver is single parent
        is_parent = spm_unit.members("is_tax_unit_head_or_spouse", period)
        is_single_parent = spm_unit.sum(is_parent) == 1
        # Factor 36: Experiencing poverty (<=200% FPL)
        is_poverty = person("il_pi_is_poverty", period)

        lower_priority_factors = [
            is_teen_parent_family,
            is_single_parent,
            is_poverty,
        ]

        return sum([f.astype(int) for f in lower_priority_factors]) * 10



from policyengine_us.model_api import *


class employer_ss_tax_income_tax_revenue(Variable):
    value_type = float
    entity = TaxUnit
    definition_period = YEAR
    label = "Income tax revenue from taxing employer SS contributions"
    documentation = (
        "Marginal income tax revenue attributable to including employer "
        "Social Security contributions in taxable income. This revenue "
        "is credited to the OASDI trust funds."
    )
    unit = USD

    def formula(tax_unit, period, parameters):
        """
        Calculate OASDI trust fund revenue from employer SS taxation.

        Method:
        1. Get current income_tax (with BOTH SS + Medicare in AGI from reform)
        2. Branch: Neutralize employer_social_security_tax, recalculate income_tax
        3. SS revenue = current_tax - tax_without_ss

        This isolates the marginal tax contribution from employer SS by
        neutralizing the source variable, letting the reform formula
        naturally exclude SS when calculating gross income.
        """
        sim = tax_unit.simulation

        # Check if reform is active
        p = parameters(period).gov.contrib.crfb.tax_employer_payroll_tax
        if not p.in_effect:
            return tax_unit.empty_array()

        # Current tax (with reform - both SS and Medicare in AGI)
        tax_full = tax_unit("income_tax", period)

        # === Branch: Tax WITHOUT employer SS (neutralize source) ===
        branch = sim.get_branch("without_employer_ss", clone_system=True)

        # Neutralize the source variable - reform formula will pick up 0 for SS
        branch.tax_benefit_system.neutralize_variable(
            "employer_social_security_tax"
        )

        # Clear cached calculations
        input_vars = set(branch.input_variables)
        for var_name in list(branch.tax_benefit_system.variables.keys()):
            if var_name not in input_vars:
                try:
                    branch.delete_arrays(var_name)
                except:
                    pass

        tax_without_ss = branch.tax_unit("income_tax", period)
        del sim.branches["without_employer_ss"]

        # SS revenue = tax with full reform - tax without SS
        return max_(tax_full - tax_without_ss, 0)



from policyengine_us.model_api import *


class wi_works_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Wisconsin Works income eligible"
    definition_period = MONTH
    reference = (
        "https://dcf.wisconsin.gov/manuals/w-2-manual/Production/03/03.2.1_115_Percent_Gross_Income_Test.htm",
        "https://docs.legis.wisconsin.gov/code/admin_code/dcf/101_199/101/09/3/a",
    )
    defined_for = StateCode.WI

    def formula(spm_unit, period, parameters):
        countable_income = spm_unit("wi_works_countable_income", period)
        fpg = spm_unit("spm_unit_fpg", period)
        p = parameters(period).gov.states.wi.dcf.works.income_limit
        income_limit = fpg * p.rate
        return countable_income <= income_limit



from policyengine_us.model_api import *


class il_pfa(Variable):
    value_type = float
    entity = Person
    label = "Illinois Preschool For All (PFA) benefit"
    definition_period = YEAR
    unit = USD
    reference = (
        "https://www.isbe.net/pages/preschool-for-all.aspx",
        "https://gov-pritzker-newsroom.prezly.com/gov-pritzker-announces-5150-new-preschool-seats-through-smart-start-initiative",
    )
    defined_for = "il_pfa_eligible"
    adds = ["gov.states.il.isbe.pfa.benefit.amount"]



import pytest

DATASETS = [
    "hf://policyengine/policyengine-us-data/cps_2023.h5",
    "hf://policyengine/policyengine-us-data/enhanced_cps_2024.h5",
]

YEARS = list(range(2024, 2026))


@pytest.mark.parametrize("dataset", DATASETS)
@pytest.mark.parametrize("year", YEARS)
def test_microsim_runs(dataset: str, year: int):
    import numpy as np
    from policyengine_us import Microsimulation

    sim = Microsimulation(dataset=dataset)
    sim.subsample(1_000)
    hnet = sim.calc("household_net_income", period=year)
    assert not hnet.isna().any(), "Some households have NaN net income."
    # Deciles are 1-10, with -1 for negative income.
    DECILES = [
        "household_income_decile",
        "spm_unit_income_decile",
        "income_decile",
    ]
    for decile_var in DECILES:
        decile = sim.calc(decile_var)
        assert np.all(decile >= -1) and np.all(
            decile <= 10
        ), f"{decile_var} out of bounds."

    # Check that the microsim calculates important variables as nonzero in current year.
    for var in ["employment_income", "self_employment_income"]:
        assert sim.calc(var, period=2024).sum() > 0, f"{var} is zero in 2024."


def test_county_persists_across_periods():
    """Test that county values persist when calculating for different periods.

    When running over a dataset that has county pre-stored, the county formula
    should return the stored value regardless of the period requested, since
    county is a time-invariant geographic variable.

    Uses the NYC dataset which has county explicitly stored (unlike cps_2023
    which doesn't store county).
    """
    import numpy as np
    from policyengine_us import Microsimulation

    sim = Microsimulation(
        dataset="hf://policyengine/policyengine-us-data/cities/NYC.h5"
    )
    sim.subsample(100)

    # Verify county is pre-stored in this dataset
    holder = sim.get_holder("county")
    known_periods = holder.get_known_periods()
    assert len(known_periods) > 0, "NYC dataset should have county pre-stored"

    # Get county for a period different from the stored period
    # NYC dataset stores county for 2023, so request 2025
    county_2025 = sim.calculate("county", period=2025).values

    # Get the stored county value for comparison
    stored_period = known_periods[0]
    county_stored = sim.calculate("county", period=stored_period).values

    # Counties should be identical - the 2025 request should return stored value
    assert np.array_equal(
        county_stored, county_2025
    ), "County values should persist across periods when running over a dataset"

    # Verify we got NYC counties, not Albany (the bug we're fixing)
    from policyengine_us.variables.household.demographic.geographic.county.county_enum import (
        County,
    )

    assert not np.any(
        county_2025 == County.ALBANY_COUNTY_NY.index
    ), "Should not fall back to Albany county"



from policyengine_us.model_api import *


class ct_tfa_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Connecticut Temporary Family Assistance (TFA) countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"
    defined_for = StateCode.CT

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ct.dss.tfa.income.deduction
        total_unearned_income = add(
            spm_unit, period, ["tanf_gross_unearned_income"]
        )
        # Get child support - PolicyEngine handles period conversion automatically
        child_support = add(spm_unit, period, ["child_support_received"])
        child_support_deduction = min_(child_support, p.child_support)

        return max_(0, total_unearned_income - child_support_deduction)



from policyengine_us.model_api import *


class mn_mfip_countable_earned_income_for_eligibility(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP countable earned income for eligibility"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/142G.16#stat.142G.16.1",
        "https://www.revisor.mn.gov/statutes/cite/256P.03#stat.256P.03.2",
    )
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.16, Subd. 1(b)(2):
        # For initial eligibility, dependent care costs must be deducted
        # from gross earned income before the $65 and 50% disregard.
        # Since dependent care is subtracted before 50%, its effect is halved.
        p = parameters(
            period
        ).gov.states.mn.dcyf.mfip.income.earned_income_disregard
        benefit_earned = spm_unit("mn_mfip_countable_earned_income", period)
        dependent_care = spm_unit("mn_mfip_dependent_care_deduction", period)
        # 50% disregard rate means dependent care effect is multiplied by 0.5
        dependent_care_adjustment = dependent_care * (1 - p.rate)
        return max_(benefit_earned - dependent_care_adjustment, 0)



from policyengine_us.model_api import *


def smi(unit_size, state, period, parameters):
    p = parameters(period).gov.hhs.smi
    size_threshold = p.additional_person_threshold
    capped_size = clip(unit_size - 1, 0, size_threshold - 1)
    extra_persons = max_(unit_size - size_threshold, 0)
    size_adjustment = (
        p.household_size_adjustment.first_person
        + p.household_size_adjustment.second_to_sixth_person * capped_size
        + p.household_size_adjustment.additional_person * extra_persons
    )
    return p.amount[state] * size_adjustment


class hhs_smi(Variable):
    value_type = float
    entity = SPMUnit
    label = "State Median Income (HHS)"
    documentation = "SPM unit's median income as defined by the Department of Health and Human Services, based on their state and size"
    definition_period = YEAR
    unit = USD

    def formula(spm_unit, period, parameters):
        size = spm_unit("spm_unit_size", period)
        state = spm_unit.household("state_code_str", period)
        return smi(size, state, period, parameters)



from policyengine_us.model_api import *


class nj_wfnj_countable_earned_income_person(Variable):
    value_type = float
    entity = Person
    label = "New Jersey WFNJ countable earned income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.NJ
    reference = (
        "https://www.law.cornell.edu/regulations/new-jersey/N-J-A-C-10-90-3-8"
    )

    def formula(person, period, parameters):
        p = parameters(period).gov.states.nj.njdhs.wfnj.income
        gross_earned = person("tanf_gross_earned_income", period)
        month = period.start.month

        # Calendar month proxy for enrollment months (assuming ≥20 hrs/week).
        # N.J.A.C. 10:90-3.8 provides tiered disregards based on months enrolled.
        # Use calendar months as a proxy:
        # - January (month 1): 100% disregard
        # - February-July (months 2-7): 75% disregard
        # - August-December (months 8-12): 50% disregard
        disregard_rate = p.earned_income_disregard.calc(month)
        return gross_earned * (1 - disregard_rate)



from policyengine_us.model_api import *


class il_pi_is_low_income(Variable):
    value_type = bool
    entity = Person
    label = "Low income for Illinois PI (Factor 6, 25 points)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=2",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        spm_unit = person.spm_unit
        # Factor 6 (25 pts): Family income at or below 100% FPL.
        p = parameters(period).gov.states.il.isbe.pi.eligibility.income
        fpg = spm_unit("spm_unit_fpg", period)
        income = spm_unit("il_isbe_countable_income", period)
        threshold = fpg * p.low_income_rate
        return income <= threshold



from policyengine_us.model_api import *


class il_hbwd_disability_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Illinois Health Benefits for Workers with Disabilities disability eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/illinois/Ill-Admin-Code-tit-89-SS-120.510",
        "https://hfs.illinois.gov/medicalprograms/hbwd/eligibility.html",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Must meet Social Security Administration's definition of disability.
        # Per DB101 Illinois "For HBWD, Social Security's disability
        # rules related to earned income do not apply" - so we use is_disabled
        # (medical definition) instead of is_ssi_disabled (which includes SGA test).
        # See: https://il.db101.org/il/programs/health_coverage/how_health/program2b.htm
        # SSA recognizes blindness as a category of disability (SSI has Aged, Blind, Disabled).
        is_disabled = person("is_disabled", period.this_year)
        is_blind = person("is_blind", period)
        # Or currently receiving SSDI
        receives_ssdi = person("social_security_disability", period) > 0
        return is_disabled | is_blind | receives_ssdi



from policyengine_us.model_api import *


class il_pfae_basic_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Meets basic eligibility for Illinois PFAE (age and income)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/pages/preschool-for-all.aspx",
        "https://www.isbe.net/Documents/pdg-eg-grant-enrollment-form.pdf",
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        # Basic eligibility: age 3-5 and income <= 400% FPL
        age_eligible = person("il_pfae_age_eligible_child", period)
        income_eligible = person("il_isbe_income_eligible", period)
        return age_eligible & income_eligible



from policyengine_us.model_api import *


class ok_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Oklahoma TANF income eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/oklahoma/OAC-340-10-3-27"
    )
    defined_for = StateCode.OK

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ok.dhs.tanf.income

        # Per OAC 340:10-3-27: Gross income limit = 185% of State Standard of Need
        unit_size = spm_unit("spm_unit_size", period)
        need_standard = p.need_standard.calc(unit_size)
        gross_income_limit = need_standard * p.gross_income_limit_rate

        # Gross income test uses gross income (before disregards)
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )

        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class nj_wfnj_gross_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Jersey WFNJ gross income"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.NJ
    reference = (
        "https://www.law.cornell.edu/regulations/new-jersey/N-J-A-C-10-90-3-3",
    )

    adds = [
        "tanf_gross_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class il_pi_highest_priority_score(Variable):
    value_type = int
    entity = Person
    label = "Illinois PI highest priority score (50 pts factors)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/Documents/Prevention-Initiative-Eligibility-Form.pdf#page=2",
    )
    defined_for = "il_pi_demographic_eligible"

    def formula(person, period, parameters):
        household = person.household

        # Highest priority factors (50 pts each):
        # Factor 1: Homeless (McKinney-Vento)
        is_homeless = household("is_homeless", period)
        # Factor 2: Youth in Care (foster care, child welfare)
        is_in_foster_care = person("is_in_foster_care", period)
        # Factor 3: Developmental delay
        has_developmental_delay = person("has_developmental_delay", period)
        # Factor 4: Deep poverty (<=50% FPL) and/or receiving TANF
        is_deep_poverty = person("il_pi_is_deep_poverty", period)
        # Factor 5: Non-English speaking home
        is_non_english_home = household("is_non_english_speaking_home", period)

        highest_priority_factors = [
            is_homeless,
            is_in_foster_care,
            has_developmental_delay,
            is_deep_poverty,
            is_non_english_home,
        ]

        return sum([f.astype(int) for f in highest_priority_factors]) * 50



from policyengine_us.model_api import *


class wv_works_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "West Virginia WV Works countable income"
    unit = USD
    definition_period = MONTH
    reference = "https://bfa.wv.gov/media/2766/download?inline#page=589"
    defined_for = StateCode.WV

    # Step 7: Add together the total countable earned and unearned income
    adds = [
        "wv_works_countable_earned_income",
        "wv_works_countable_unearned_income",
    ]



from policyengine_us.model_api import *


class ne_adc_need_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Nebraska ADC standard of need"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://nebraskalegislature.gov/laws/statutes.php?statute=43-513",
        "https://dhhs.ne.gov/Pages/Title-468.aspx",
    )
    defined_for = StateCode.NE

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ne.dhhs.adc
        size = spm_unit("spm_unit_size", period.this_year)
        # Parameter table defines sizes 1-10; for larger sizes, add increment per person
        max_table_size = p.max_unit_size
        capped_size = min_(size, max_table_size)
        base_amount = p.benefit.standard_of_need.amount[capped_size]
        additional_persons = max_(size - max_table_size, 0)
        additional_amount = (
            additional_persons * p.benefit.standard_of_need.additional_person
        )
        return base_amount + additional_amount



from policyengine_us.model_api import *


class ri_works_dependent_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Rhode Island Works dependent care deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/rhode-island/218-RICR-20-00-2.15",
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
    )
    defined_for = StateCode.RI

    def formula(spm_unit, period, parameters):
        # Per 218-RICR-20-00-2.15.5(B)(2): Dependent care expenses deducted
        # up to $200/month for children under age 2
        # up to $175/month for children age 2 and older
        p = parameters(period).gov.states.ri.dhs.works.income.dependent_care
        person = spm_unit.members

        is_dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)

        max_per_dependent = p.amount.calc(age)
        total_max_deduction = spm_unit.sum(max_per_dependent * is_dependent)

        childcare_expenses = spm_unit("childcare_expenses", period)

        return min_(childcare_expenses, total_max_deduction)



from policyengine_us.model_api import *


class hi_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Hawaii TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=19",
    )
    defined_for = StateCode.HI

    adds = ["hi_tanf_countable_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class ga_military_retirement_exclusion_eligible_person(Variable):
    value_type = bool
    entity = Person
    label = "Eligible person for the Georgia military retirement exclusion"
    definition_period = YEAR
    reference = (
        "https://dor.georgia.gov/document/document/2022-it-511-individual-income-tax-booklet/download",  # Page 16
        "https://advance.lexis.com/documentpage/?pdmfid=1000516&crid=fb5db531-a80f-4790-bddb-eefc8327ef60&config=00JAA1MDBlYzczZi1lYjFlLTQxMTgtYWE3OS02YTgyOGM2NWJlMDYKAFBvZENhdGFsb2feed0oM9qoQOMCSJFX5qkd&pddocfullpath=%2Fshared%2Fdocument%2Fstatutes-legislation%2Furn%3AcontentItem%3A65D2-CDH3-CGX8-044N-00008-00&pdcontentcomponentid=234186&pdteaserkey=sr1&pditab=allpods&ecomp=8s65kkk&earg=sr1&prid=66f02b0a-c5ae-4162-9535-127751546807",
    )
    defined_for = StateCode.GA

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.states.ga.tax.income.agi.exclusions.military_retirement
        head_or_spouse = person("is_tax_unit_head_or_spouse", period)
        age_eligible = person("age", period) < p.age_limit
        return head_or_spouse & age_eligible



from policyengine_us.model_api import *


class ar_tea_maximum_benefit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Arkansas TEA maximum benefit"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = StateCode.AR

    def formula(spm_unit, period, parameters):
        # Per 208.00.13 Ark. Code R. Section 001, Section 6.1.1
        p = parameters(period).gov.states.ar.dhs.tea.payment_standard
        unit_size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(unit_size, p.max_unit_size)
        return p.amount[capped_size]



from policyengine_us.model_api import *


class tn_ff_child_care_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "Tennessee Families First child care deduction"
    unit = USD
    definition_period = MONTH
    defined_for = StateCode.TN
    reference = "https://publications.tnsosfiles.com/rules/1240/1240-01/1240-01-50.20081124.pdf#page=19"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.tn.dhs.ff.income.deductions
        person = spm_unit.members
        dependent = person("is_tax_unit_dependent", period)
        age = person("monthly_age", period)
        childcare_expenses = spm_unit("childcare_expenses", period)
        childcare_deduction_person = (
            p.child_care_deduction.calc(age) * dependent
        )
        total_childcare_deduction = spm_unit.sum(childcare_deduction_person)
        return min_(childcare_expenses, total_childcare_deduction)



from policyengine_us.model_api import *


class ak_atap_need_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP need standard"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.520"
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ak.dpa.atap.need_standard
        unit_size = spm_unit("spm_unit_size", period)

        # Check for pregnant woman alone (size 1 with pregnant person)
        is_pregnant = add(spm_unit, period, ["is_pregnant"]) > 0
        is_pregnant_woman_alone = (unit_size == 1) & is_pregnant

        # Adult-included standards: sizes 2-8, with additional amount for larger
        max_unit_size = p.max_unit_size
        capped_size = clip(unit_size, 2, max_unit_size)
        base_standard = p.amount[capped_size]

        # Add additional amount for each person beyond max table size
        extra_persons = max_(unit_size - max_unit_size, 0)
        additional = extra_persons * p.additional_person
        adult_included_standard = base_standard + additional

        # Return pregnant woman standard if alone, otherwise adult-included
        return where(
            is_pregnant_woman_alone,
            p.pregnant_woman,
            where(unit_size >= 2, adult_included_standard, 0),
        )



from policyengine_us.model_api import *


class ks_tanf_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kansas TANF countable earned income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/kansas/K-A-R-30-4-110",
        "https://content.dcf.ks.gov/ees/keesm/current/keesm7110.htm",
        "https://content.dcf.ks.gov/ees/keesm/current/keesm7224.htm",
    )
    defined_for = StateCode.KS

    def formula(spm_unit, period, parameters):
        # Per K.A.R. 30-4-110, KEESM 7110, and KEESM 7224:
        # Sum person-level earned income after $90 and 60% deductions,
        # then subtract dependent care expenses.
        #
        # Per KEESM 7224: Dependent care is applied after $90 and 60% disregards.
        # There is NO cap on dependent care deduction in Kansas TANF.
        earned_after_deductions = add(
            spm_unit, period, ["ks_tanf_earned_income_after_deductions"]
        )
        dependent_care = spm_unit(
            "spm_unit_pre_subsidy_childcare_expenses", period
        )
        return max_(earned_after_deductions - dependent_care, 0)



from policyengine_us.model_api import *


class nh_fanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "New Hampshire FANF income eligible"
    definition_period = MONTH
    reference = (
        "https://gc.nh.gov/rsa/html/xii/167/167-77-g.htm",
        "https://www.dhhs.nh.gov/fam_htm/newfam.htm",
    )
    defined_for = StateCode.NH

    def formula(spm_unit, period, parameters):
        countable_income = spm_unit("nh_fanf_countable_income", period)
        payment_standard = spm_unit("nh_fanf_payment_standard", period)
        return countable_income <= payment_standard



from policyengine_us.model_api import *


class ak_atap_gross_income_limit(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP gross income limit"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.520"
    defined_for = StateCode.AK

    def formula(spm_unit, period, parameters):
        # Per 7 AAC 45.520: Gross income limit is 185% of need standard
        p = parameters(period).gov.states.ak.dpa.atap
        need_standard = spm_unit("ak_atap_need_standard", period)
        return need_standard * p.gross_income_limit_rate



from policyengine_us.model_api import *


class ky_ktap_standard_of_need(Variable):
    value_type = float
    entity = SPMUnit
    label = "Kentucky K-TAP standard of need"
    unit = USD
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ky.dcbs.ktap.benefit
        unit_size = spm_unit("spm_unit_size", period)
        capped_size = min_(unit_size, p.max_unit_size)
        return p.standard_of_need[capped_size]



from policyengine_us.model_api import *


class or_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Oregon TANF eligible"
    definition_period = MONTH
    reference = "https://oregon.public.law/rules/oar_461-135-0070"
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        person = spm_unit.members
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        has_citizen = spm_unit.any(
            person("is_citizen_or_legal_immigrant", period)
        )
        income_eligible = spm_unit("or_tanf_income_eligible", period)
        resources_eligible = spm_unit("or_tanf_resources_eligible", period)
        return (
            demographic_eligible
            & has_citizen
            & income_eligible
            & resources_eligible
        )



from policyengine_us.model_api import *


class il_pfa_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Eligible for Illinois Preschool For All (PFA)"
    definition_period = YEAR
    reference = (
        "https://www.isbe.net/pages/preschool-for-all.aspx",
        "https://www.isbe.net/Documents/pdg-eg-grant-enrollment-form.pdf",
    )
    defined_for = StateCode.IL
    # PFA and PFAE share identical eligibility criteria. The difference is
    # service level: PFA provides half-day, PFAE provides full-day programs.
    # A child receives one or the other based on local program availability,
    # not both simultaneously.
    adds = ["il_pfae_eligible"]



from policyengine_us.model_api import *


class fl_tca(Variable):
    value_type = float
    entity = SPMUnit
    label = "Florida Temporary Cash Assistance"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.leg.state.fl.us/statutes/index.cfm?App_mode=Display_Statute&URL=0400-0499/0414/Sections/0414.095.html",
        "https://www.law.cornell.edu/regulations/florida/Fla-Admin-Code-Ann-R-65A-4-220",
    )
    defined_for = "fl_tca_eligible"

    def formula(spm_unit, period, parameters):
        # Per Florida Statutes 414.095(12) and FAC 65A-4.220
        payment_standard = spm_unit("fl_tca_payment_standard", period)
        countable_income = spm_unit("fl_tca_countable_income", period)

        # Benefit = Payment Standard - Countable Income
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class is_qmb_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Qualified Medicare Beneficiary (QMB) eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/cfr/text/42/435.121",
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
    )

    def formula(person, period, parameters):
        # QMB requires income at or below 100% FPL
        p = parameters(
            period
        ).gov.hhs.medicare.savings_programs.eligibility.income

        medicare_eligible = person("is_medicare_eligible", period.this_year)
        asset_eligible = person("msp_asset_eligible", period)

        fpg = person("msp_fpg", period)
        countable_income = person("msp_countable_income", period)
        qmb_income_limit = fpg * p.qmb.fpl_limit
        income_eligible = countable_income <= qmb_income_limit

        return medicare_eligible & income_eligible & asset_eligible



from policyengine_us.model_api import *


class me_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Maine TANF resources eligible"
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://legislature.maine.gov/legis/bills/getPDF.asp?paper=HP0892&item=3&session=131",
    )
    defined_for = StateCode.ME

    def formula(spm_unit, period, parameters):
        # Per 22 M.R.S. Section 3762 and P.L. 2023 Ch. 366:
        # Resource limit is $10,000 per family
        # One vehicle per licensed driver is exempt
        p = parameters(period).gov.states.me.dhhs.tanf
        countable_resources = spm_unit("spm_unit_assets", period.this_year)
        return countable_resources <= p.resource_limit



from policyengine_us.model_api import *


class tn_ff_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Tennessee Families First payment standard"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/tennessee/Tenn-Comp-R-Regs-1240-01-50-.20"
    defined_for = StateCode.TN

    def formula(spm_unit, period, parameters):
        # NOTE: Tennessee has a Differential Grant Payment Amount (DGPA) for
        # caretakers who are age 60+, disabled, or provide care for disabled.
        # This is not currently modeled; we only return the Standard Payment
        # Amount (SPA). DGPA parameters exist for future implementation.
        p = parameters(period).gov.states.tn.dhs.ff.payment
        unit_size = spm_unit("spm_unit_size", period.this_year)
        capped_size = min_(unit_size, p.max_family_size)

        return p.standard_payment_amount[capped_size]



from policyengine_us.model_api import *
from policyengine_core.periods import period as period_


def create_ut_hb210() -> Reform:
    """
    Utah HB 210 (2026) - Tax penalty removal bill that:
    1. Adds a taxpayer credit add-on for married filers ($543 MFS, $1,086 joint)
    2. Removes marriage penalties from phaseout thresholds (parameter changes)
    3. Repeals the Utah EITC (parameter change - set rate to 0)

    This reform implements provision #1 (the structural change).
    Provisions #2 and #3 can be modeled via parameter changes.
    """

    class ut_taxpayer_credit(Variable):
        value_type = float
        entity = TaxUnit
        label = "Utah taxpayer credit"
        unit = USD
        documentation = "Form TC-40, line 20"
        definition_period = YEAR
        defined_for = StateCode.UT
        reference = (
            "https://le.utah.gov/xcode/Title59/Chapter10/59-10-S1018.html",
            "https://le.utah.gov/~2026/bills/static/HB0210.html",
        )

        def formula(tax_unit, period, parameters):
            # Base maximum credit (6% of deductions + exemption)
            base_max = tax_unit("ut_taxpayer_credit_max", period)

            # Add the married filer add-on per HB 210 Section 59-10-1018(3)
            # The add-on is added before phaseout is applied
            filing_status = tax_unit("filing_status", period)
            p = parameters(
                period
            ).gov.contrib.states.ut.hb210.taxpayer_credit_add_on
            add_on = p.amount[filing_status]

            # Total maximum credit including add-on
            total_max = base_max + add_on

            # Apply phaseout reduction to total credit (including add-on)
            reduction = tax_unit("ut_taxpayer_credit_reduction", period)
            return max_(total_max - reduction, 0)

    class reform(Reform):
        def apply(self):
            self.update_variable(ut_taxpayer_credit)

    return reform


def create_ut_hb210_reform(parameters, period, bypass: bool = False):
    if bypass:
        return create_ut_hb210()

    p = parameters.gov.contrib.states.ut.hb210

    reform_active = False
    current_period = period_(period)

    for i in range(5):
        if p(current_period).in_effect:
            reform_active = True
            break
        current_period = current_period.offset(1, "year")

    if reform_active:
        return create_ut_hb210()
    else:
        return None


ut_hb210 = create_ut_hb210_reform(None, None, bypass=True)



from policyengine_us.model_api import *


class ri_works_earned_income_after_disregard_person(Variable):
    value_type = float
    entity = Person
    label = "Rhode Island Works earned income after disregard per person"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/rhode-island/218-RICR-20-00-2.15",
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
    )
    defined_for = StateCode.RI

    def formula(person, period, parameters):
        # Per 218-RICR-20-00-2.15: Disregard $525 + 50% of remainder
        # "This disregard is allowed for each individual who has
        # otherwise been found eligible to receive cash assistance."
        p = parameters(
            period
        ).gov.states.ri.dhs.works.income.earned_income_disregard
        gross_earned = person("tanf_gross_earned_income", period)
        after_flat_disregard = max_(gross_earned - p.amount, 0)
        return after_flat_disregard * (1 - p.rate)



from policyengine_us.model_api import *


class mi_fip_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Michigan FIP based on income"
    definition_period = MONTH
    reference = (
        "https://mdhhs-pres-prod.michigan.gov/olmweb/ex/BP/Public/BEM/518.pdf#page=5",
        "https://mdhhs-pres-prod.michigan.gov/olmweb/EX/BP/Public/BEM/520.pdf#page=3",
    )
    defined_for = StateCode.MI

    def formula(spm_unit, period, parameters):
        # BEM 518 Page 3: Financial need exists when the group passes
        # the Qualifying Deficit Test (initial) or Issuance Deficit Test (ongoing)

        enrolled = spm_unit("is_tanf_enrolled", period)
        payment_standard = spm_unit("mi_fip_payment_standard", period)

        # New applicants: Use Qualifying Deficit Test (BEM 520 Section C)
        # - Uses 20% deduction rate
        countable_income_for_eligibility = spm_unit(
            "mi_fip_countable_income_for_eligibility", period
        )
        passes_qualifying_test = (
            countable_income_for_eligibility < payment_standard
        )

        # Enrolled recipients: Use Issuance Deficit Test (BEM 520 Section D)
        # - Uses 50% deduction rate (more generous)
        countable_income_for_benefit = spm_unit(
            "mi_fip_countable_income_for_benefit", period
        )
        passes_issuance_test = countable_income_for_benefit < payment_standard

        # Return appropriate test based on enrollment status
        return where(enrolled, passes_issuance_test, passes_qualifying_test)



from policyengine_us.model_api import *


class hi_tanf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Hawaii TANF"
    definition_period = MONTH
    reference = (
        "https://humanservices.hawaii.gov/wp-content/uploads/2024/12/Hawaii_TANF_State_Plan_Signed_Certified-Eff_20231001.pdf#page=6",
    )
    defined_for = StateCode.HI

    def formula(spm_unit, period, parameters):
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        gross_income_eligible = spm_unit(
            "hi_tanf_gross_income_eligible", period
        )
        # NOTE: Hawaii disregards assets since April 18, 2013
        return (
            demographic_eligible & immigration_eligible & gross_income_eligible
        )



from policyengine_us.model_api import *


class oh_agi_person(Variable):
    value_type = float
    entity = Person
    label = "Ohio adjusted gross income"
    unit = USD
    definition_period = YEAR
    reference = (
        "https://codes.ohio.gov/ohio-revised-code/section-5747.055",
        "https://tax.ohio.gov/static/forms/ohio_individual/individual/2021/pit-it1040-booklet.pdf#page=20",
    )
    defined_for = StateCode.OH

    adds = ["adjusted_gross_income_person", "oh_additions"]

    subtracts = ["oh_deductions"]



from policyengine_us.model_api import *


class is_born_outside_us(Variable):
    value_type = bool
    entity = Person
    label = "Was born outside the United States"
    definition_period = YEAR
    documentation = "Whether the person was born outside the United States"



from policyengine_us.model_api import *


class in_tanf_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Indiana TANF payment standard"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://iga.in.gov/laws/2025/ic/titles/12/#12-14-2-5",
        "https://iar.iga.in.gov/code/2026/470/10.3#470-10.3-4-3",
    )
    defined_for = StateCode.IN

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["in"].fssa.tanf.standard_of_need
        capped_size = min_(spm_unit("spm_unit_size", period), p.max_unit_size)
        return p.amount[capped_size]



from policyengine_us.model_api import *


class wy_power(Variable):
    value_type = float
    entity = SPMUnit
    label = "Wyoming POWER benefit"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://dfs.wyo.gov/about/policy-manuals/snap-and-power-policy-manual/",
        "https://dfs.wyo.gov/accordions/snap-and-power-policy-manual-1100-extended-menu/",
    )
    defined_for = "wy_power_eligible"

    def formula(spm_unit, period, parameters):
        # Per Section 1101: Benefit = Payment Standard - Countable Income
        payment_standard = spm_unit("wy_power_payment_standard", period)
        countable_income = spm_unit("wy_power_countable_income", period)
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class oh_owf_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Ohio OWF eligibility"
    definition_period = MONTH
    defined_for = StateCode.OH
    reference = "https://codes.ohio.gov/ohio-revised-code/section-5107.10"

    def formula(spm_unit, period, parameters):
        person = spm_unit.members

        # Must meet demographic requirements (minor child with parent/relative
        # OR pregnant woman at least 6 months pregnant)
        # Use federal demographic eligibility
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)

        # Must have at least one citizen or legal immigrant
        # Use federal immigration eligibility
        immigration_status_eligible = spm_unit.any(
            person("is_citizen_or_legal_immigrant", period)
        )

        # Must meet income requirements
        # (oh_owf_income_eligible handles both initial and ongoing tests)
        income_eligible = spm_unit("oh_owf_income_eligible", period)

        # Ohio has NO resource limits per working_references.md:
        # "No Resource Test: Resources such as a car or home ownership
        # are NOT considered in determining eligibility"

        # All requirements must be met
        return (
            demographic_eligible
            & immigration_status_eligible
            & income_eligible
        )



from policyengine_us.model_api import *


class pell_grant(Variable):
    value_type = float
    entity = Person
    label = "Pell Grant"
    definition_period = YEAR
    unit = USD

    def formula(person, period, parameters):
        coa = person("cost_of_attending_college", period)
        p = parameters(period).gov.ed.pell_grant

        if p.uses_sai:
            sai = person("pell_grant_sai", period)
            eligibility = person("pell_grant_eligibility_type", period)
            unbounded = coa - sai
            amount = where(unbounded < p.amount.min, 0, unbounded)
            uncapped = select(
                [
                    eligibility == eligibility.possible_values.MAXIMUM,
                    eligibility == eligibility.possible_values.MINIMUM,
                ],
                [p.amount.max, amount],
                default=0,
            )
        else:
            months_in_school = person("pell_grant_months_in_school", period)
            efc = person("pell_grant_efc", period)
            unbounded = coa - efc
            amount = where(unbounded < p.amount.min, 0, unbounded)
            uncapped = amount * (months_in_school / p.months_in_school_year)

        max_amount = min_(coa, p.amount.max)
        return min_(max_amount, uncapped)



from policyengine_us.model_api import *


class sd_tanf_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "South Dakota TANF payment standard"
    unit = USD
    definition_period = MONTH
    reference = "https://sdlegislature.gov/Rules/Administrative/67:10:05:03"
    defined_for = StateCode.SD

    def formula(spm_unit, period, parameters):
        size = spm_unit("spm_unit_size", period.this_year)
        p = parameters(period).gov.states.sd.dss.tanf.payment_standard
        is_shared = spm_unit("sd_tanf_is_shared_living", period)

        max_table_size = p.max_table_size
        base_size = min_(size, max_table_size)
        independent_amount = p.independent_living[base_size]
        shared_amount = p.shared_living[base_size]

        extra_persons = max_(size - max_table_size, 0)
        increment = p.additional_person_increment * extra_persons

        base_amount = where(is_shared, shared_amount, independent_amount)
        return base_amount + increment



from policyengine_us.model_api import *


class or_tanf_countable_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Oregon TANF countable income eligible"
    definition_period = MONTH
    reference = (
        "https://secure.sos.state.or.us/oard/viewSingleRule.action?ruleVrsnRsn=268185",
        "https://oregon.public.law/rules/oar_461-155-0030",
    )
    defined_for = StateCode.OR

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states["or"].odhs.tanf
        size = spm_unit("spm_unit_size", period.this_year)
        max_size = p.maximum_need_group_size
        size_capped = min_(size, max_size)
        additional_people = max_(size - max_size, 0)

        countable_income = spm_unit("or_tanf_countable_income", period)
        countable_limit = (
            p.income.countable_income_limit.amount[size_capped]
            + additional_people
            * p.income.countable_income_limit.additional_person
        )
        return countable_income < countable_limit



from policyengine_us.model_api import *


class ak_atap_countable_earned_income_person(Variable):
    value_type = float
    entity = Person
    label = "Alaska ATAP countable earned income per person"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.480"
    defined_for = StateCode.AK

    def formula(person, period, parameters):
        # Per 7 AAC 45.480: Deductions depend on applicant status
        # (a)(1) New applicants: $90 flat deduction
        # (a)(2) + (b) Continuing recipients: $150 flat + percentage
        p = parameters(period).gov.states.ak.dpa.atap.income
        gross_earned = person("tanf_gross_earned_income", period)
        is_enrolled = person.spm_unit("is_tanf_enrolled", period)

        # New applicants: $90 flat deduction only
        new_applicant_countable = max_(
            gross_earned - p.deductions.initial_work_deduction, 0
        )

        # Continuing recipients: $150 flat + percentage of remainder
        # NOTE: Alaska uses tiered deductions based on months receiving assistance:
        #   Tier 1 (months 1-12): $150 + 33%
        #   Tier 2 (months 13-24): $150 + 25%
        #   Tier 3 (months 25-36): $150 + 20%
        #   Tier 4 (months 37-48): $150 + 15%
        #   Tier 5 (months 49-60): $150 + 10%
        #   Beyond 60 months: $150 only (0%)
        #   Only Tier 1 is modeled.
        after_flat = max_(gross_earned - p.work_incentive.flat, 0)
        percent_disregard = after_flat * p.work_incentive.rate
        continuing_countable = max_(
            gross_earned - p.work_incentive.flat - percent_disregard, 0
        )

        return where(
            is_enrolled, continuing_countable, new_applicant_countable
        )



from policyengine_us.model_api import *


class ca_capp_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for California Alternative Payment Program"
    definition_period = MONTH
    defined_for = StateCode.CA
    reference = "https://leginfo.legislature.ca.gov/faces/codes_displaySection.xhtml?sectionNum=10271.&lawCode=WIC"

    def formula(spm_unit, period, parameters):
        # CAPP is for families who have NEVER received CalWORKs
        ever_received = spm_unit("was_calworks_recipient", period.this_year)

        # Not currently receiving CalWORKs cash aid
        currently_receiving = spm_unit("ca_tanf", period) > 0

        not_calworks_recipient = ~ever_received & ~currently_receiving

        # Income must be at or below 85% SMI
        income_eligible = spm_unit("ca_child_care_income_eligible", period)

        # Must have a need for child care (work requirement)
        need_eligible = spm_unit(
            "ca_calworks_child_care_meets_work_requirement", period.this_year
        )

        # Must have age-eligible children
        age_eligible = spm_unit("ca_calworks_child_care_age_eligible", period)

        # NOTE: CAPP is subject to funding availability and waitlists,
        # which cannot be modeled in PolicyEngine. We assume enrollment.
        return (
            not_calworks_recipient
            & income_eligible
            & need_eligible
            & age_eligible
        )



from policyengine_us.model_api import *


class nm_works_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Mexico Works countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.srca.nm.gov/parts/title08/08.102.0520.html"
    defined_for = StateCode.NM

    def formula(spm_unit, period, parameters):
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        earned_deduction = add(
            spm_unit, period, ["nm_works_earned_income_deduction_person"]
        )
        childcare_deduction = spm_unit("nm_works_childcare_deduction", period)
        return max_(gross_earned - earned_deduction - childcare_deduction, 0)



from policyengine_us.model_api import *


class ut_fep(Variable):
    value_type = float
    entity = SPMUnit
    label = "Utah Family Employment Program benefit"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = "ut_fep_eligible"

    def formula(spm_unit, period, parameters):
        # Per R986-200-246: Benefit = payment standard - countable income
        payment_standard = spm_unit("ut_fep_payment_standard", period)
        countable_income = spm_unit("ut_fep_countable_income", period)
        return max_(payment_standard - countable_income, 0)



from policyengine_us.model_api import *


class il_ihwap_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Illinois IHWAP eligible"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://www.law.cornell.edu/uscode/text/42/6862#7",
        "https://dceo.illinois.gov/communityservices/homeweatherization.html",
    )

    def formula(spm_unit, period, parameters):
        income_eligible = spm_unit("il_ihwap_income_eligible", period)
        categorically_eligible = spm_unit(
            "il_ihwap_categorically_eligible", period
        )
        return income_eligible | categorically_eligible



from policyengine_us.model_api import *


class il_aabd_ssi_income(Variable):
    value_type = float
    entity = Person
    definition_period = MONTH
    unit = USD
    label = "SSI income for IL AABD calculation"
    documentation = (
        "SSI amount used in IL AABD unearned income calculation. "
        "By default, uses calculated ssi. When il_aabd_use_reported_ssi "
        "is True, uses ssi_reported instead."
    )
    defined_for = StateCode.IL

    def formula(person, period, parameters):
        use_reported = person("il_aabd_use_reported_ssi", period.this_year)
        reported = person("ssi_reported", period)
        calculated = person("ssi", period)
        return where(use_reported, reported, calculated)



from policyengine_us.model_api import *


class ak_atap_gross_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Alaska ATAP gross income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/alaska/7-AAC-45.470"
    defined_for = StateCode.AK

    adds = ["tanf_gross_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class nm_works_child_support_deduction(Variable):
    value_type = float
    entity = SPMUnit
    label = "New Mexico Works child support deduction"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.srca.nm.gov/parts/title08/08.102.0520.html",
        "https://www.hca.nm.gov/2023/01/10/human-services-department-to-pass-through-more-money-to-low-income-families/",
    )
    defined_for = StateCode.NM

    def formula(spm_unit, period, parameters):
        # Per 8.102.520.9.J NMAC:
        # - $50 disregard on child support received
        # - Passthrough: $100 for 1 child, $200 for 2+ children (since Jan 2023)
        p = parameters(
            period
        ).gov.states.nm.hca.nm_works.income.deductions.child_support

        child_support_received = add(
            spm_unit, period, ["child_support_received"]
        )
        num_children = spm_unit("spm_unit_count_children", period.this_year)

        total_deduction = p.disregard + p.passthrough.calc(num_children)
        return min_(child_support_received, total_deduction)



from policyengine_us.model_api import *


class has_developmental_delay(Variable):
    value_type = bool
    entity = Person
    label = "Has a developmental delay"
    definition_period = YEAR
    documentation = "Whether the child has a screening-indicated developmental delay or disability"



from policyengine_us.model_api import *


class nd_tanf_countable_unearned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "North Dakota TANF countable unearned income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.nd.gov/dhs/policymanuals/40019/400_19_55_25.htm"
    defined_for = StateCode.ND

    # Child support received assigned to Child Support Division is excluded.
    # Result cannot be negative since child_support_received is a component
    # of tanf_gross_unearned_income.
    adds = ["tanf_gross_unearned_income"]
    subtracts = ["child_support_received"]



from policyengine_us.model_api import *


class wv_works_countable_earned_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "West Virginia WV Works countable earned income"
    unit = USD
    definition_period = MONTH
    reference = "https://bfa.wv.gov/media/2766/download?inline#page=586"
    defined_for = StateCode.WV

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.wv.dhhr.works.income
        # Step 1: Add together the countable gross earned income
        gross_earned = add(spm_unit, period, ["tanf_gross_earned_income"])
        # Step 2: Subtract 40% (Earned Income Disregard)
        after_disregard = gross_earned * (1 - p.earned_income_disregard.rate)
        # Step 3: Subtract Dependent Care Deduction (no maximum, per Section 4.5.2.A.2)
        dependent_care = spm_unit("childcare_expenses", period)
        return max_(after_disregard - dependent_care, 0)



from policyengine_us.model_api import *


class mo_tanf_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Missouri TANF income eligibility (passes all three income tests)"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
    )
    defined_for = StateCode.MO

    def formula(spm_unit, period, parameters):
        gross = spm_unit("mo_tanf_gross_income_eligible", period)
        need = spm_unit("mo_tanf_standard_of_need_test", period)
        pct = spm_unit("mo_tanf_percentage_of_need_test", period)
        return gross & need & pct



from policyengine_us.model_api import *


class ky_ktap_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Kentucky K-TAP due to income"
    definition_period = MONTH
    reference = "https://apps.legislature.ky.gov/law/kar/titles/921/002/016/"
    defined_for = StateCode.KY

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ky.dcbs.ktap.eligibility
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )
        standard_of_need = spm_unit("ky_ktap_standard_of_need", period)
        gross_income_limit = standard_of_need * p.gross_income_limit_rate
        return gross_income <= gross_income_limit



from policyengine_us.model_api import *


class me_tanf_countable_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Maine TANF countable income"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.mainelegislature.org/legis/statutes/22/title22sec3762.html",
        "https://www.law.cornell.edu/regulations/maine/10-144-C-M-R-ch-331",
    )
    defined_for = StateCode.ME

    adds = [
        "me_tanf_countable_earned_income",
        "me_tanf_countable_unearned_income",
    ]



from policyengine_us.model_api import *


class mn_mfip_transitional_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP Transitional Standard"
    unit = USD
    definition_period = MONTH
    reference = "https://www.house.mn.gov/hrd/pubs/ss/ssmfip.pdf#page=5"
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.17, Subd. 5:
        # Transitional Standard is the maximum MFIP grant by family size.
        p = parameters(period).gov.states.mn.dcyf.mfip.transitional_standard
        size = spm_unit("spm_unit_size", period)
        capped_size = min_(size, p.max_unit_size)
        base_amount = p.amount[capped_size]
        additional_persons = max_(size - p.max_unit_size, 0)
        return base_amount + additional_persons * p.additional_person



from policyengine_us.model_api import *


class mo_tanf_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Missouri TANF gross income eligibility"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/missouri/13-CSR-40-2-310",
        "https://dssmanuals.mo.gov/temporary-assistance-case-management/0210-010-05-185/",
    )
    defined_for = StateCode.MO

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.mo.dss.tanf
        gross_income = add(
            spm_unit,
            period,
            ["tanf_gross_earned_income", "tanf_gross_unearned_income"],
        )
        standard_of_need = spm_unit("mo_tanf_standard_of_need", period)
        limit = standard_of_need * p.income_limit.rate
        return gross_income < limit



from policyengine_us.model_api import *


class ct_tfa_countable_earned_income_at_application(Variable):
    value_type = float
    entity = Person
    label = "Connecticut Temporary Family Assistance (TFA) countable earned income at application per person"
    unit = USD
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=10"
    defined_for = StateCode.CT

    def formula(person, period, parameters):
        p = parameters(
            period
        ).gov.states.ct.dss.tfa.income.deduction.new_applicant
        gross_earned_income = person("tanf_gross_earned_income", period)

        return max_(gross_earned_income - p.amount, 0)



from policyengine_us.model_api import *


class pa_tanf(Variable):
    value_type = float
    entity = SPMUnit
    label = "Pennsylvania TANF"
    unit = USD
    definition_period = MONTH
    defined_for = "pa_tanf_eligible"
    reference = (
        "https://www.pa.gov/agencies/dhs/resources/cash-assistance/tanf"
    )

    def formula(spm_unit, period, parameters):
        maximum_benefit = spm_unit("pa_tanf_maximum_benefit", period)
        countable_income = spm_unit("pa_tanf_countable_income", period)

        return max_(maximum_benefit - countable_income, 0)



from policyengine_us.model_api import *


class ut_fep_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Utah Family Employment Program due to income"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    def formula(spm_unit, period, parameters):
        # Utah uses two-tier income test per R986-200-238
        gross_test = spm_unit("ut_fep_gross_income_eligible", period)
        net_test = spm_unit("ut_fep_net_income_eligible", period)
        return gross_test & net_test



from policyengine_us.model_api import *


class de_tanf_gross_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Delaware TANF gross income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/delaware/16-Del-Admin-Code-SS-4000-4008"
    defined_for = StateCode.DE

    adds = ["tanf_gross_earned_income", "tanf_gross_unearned_income"]



from policyengine_us.model_api import *


class ct_tfa_payment_standard(Variable):
    value_type = float
    entity = SPMUnit
    label = "Connecticut Temporary Family Assistance (TFA) payment standard"
    unit = USD
    definition_period = MONTH
    reference = "https://portal.ct.gov/dss/-/media/departments-and-agencies/dss/state-plans-and-federal-reports/tanf-state-plan/ct-tanf-state-plan-2024---2026---41524-amendment.pdf#page=53"
    defined_for = StateCode.CT

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.ct.dss.tfa.payment
        size = spm_unit("spm_unit_size", period.this_year)
        capped_unit_size = min_(size, p.max_unit_size)

        return p.amount[capped_unit_size]



from policyengine_us.model_api import *


class il_scretd_deferral_amount(Variable):
    value_type = float
    entity = TaxUnit
    label = "Illinois Senior Citizens Real Estate Tax Deferral amount"
    unit = USD
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = (
        "https://www.ilga.gov/legislation/ilcs/documents/032000300K3.htm",
        "https://tax.illinois.gov/research/publications/pio-64.html",
    )

    def formula(tax_unit, period, parameters):
        # Note: This is a loan, not a benefit. The deferred amount accrues
        # interest (3% simple for 2023+, 6% prior) and must be repaid upon
        # sale, transfer, or death. Interest accumulation is not modeled.
        p = parameters(period).gov.states.il.idor.scretd
        eligible = tax_unit("il_scretd_eligible", period)
        real_estate_taxes = add(tax_unit, period, ["real_estate_taxes"])
        capped_deferral = min_(real_estate_taxes, p.max_annual_deferral)
        return where(eligible, capped_deferral, 0)



from policyengine_us.model_api import *


class mn_mfip_family_wage_level(Variable):
    value_type = float
    entity = SPMUnit
    label = "Minnesota MFIP Family Wage Level"
    unit = USD
    definition_period = MONTH
    reference = (
        "https://www.revisor.mn.gov/statutes/cite/142G.02#stat.142G.02.42"
    )
    defined_for = StateCode.MN

    def formula(spm_unit, period, parameters):
        # Per MN Stat. 142G.17, Subd. 7:
        # Family Wage Level = 110% of Transitional Standard.
        # Used for both eligibility testing and benefit calculation.
        p = parameters(period).gov.states.mn.dcyf.mfip.income
        transitional_standard = spm_unit(
            "mn_mfip_transitional_standard", period
        )
        return transitional_standard * p.family_wage_level_multiplier



from policyengine_us.model_api import *


class ri_works_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Eligible for Rhode Island Works"
    definition_period = MONTH
    reference = (
        "https://rules.sos.ri.gov/Regulations/part/218-20-00-2",
        "https://dhs.ri.gov/programs-and-services/ri-works-program/eligibility-how-apply",
    )
    defined_for = StateCode.RI

    def formula(spm_unit, period, parameters):
        income_eligible = spm_unit("ri_works_income_eligible", period)
        resource_eligible = spm_unit("ri_works_resource_eligible", period)
        demographic_eligible = (
            spm_unit("is_demographic_tanf_eligible", period) > 0
        )
        return income_eligible & resource_eligible & demographic_eligible



from policyengine_us.model_api import *


class msp(Variable):
    value_type = float
    entity = Person
    unit = USD
    label = "Medicare Savings Program benefit"
    definition_period = MONTH
    reference = (
        "https://www.medicare.gov/basics/costs/help/medicare-savings-programs",
    )
    defined_for = "msp_eligible"

    adds = ["msp_benefit_value"]



from policyengine_us.model_api import *


class pa_tanf_resources_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Pennsylvania TANF resources eligibility"
    definition_period = MONTH
    defined_for = StateCode.PA
    reference = "https://www.pacodeandbulletin.gov/Display/pacode?file=/secure/pacode/data/055/chapter178/chap178toc.html"

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.pa.dhs.tanf

        household_assets = spm_unit("spm_unit_assets", period.this_year)

        return household_assets <= p.resource_limit



from policyengine_us.model_api import *


class ar_tea_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Arkansas TEA eligible"
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/arkansas/208-00-13-Ark-Code-R-SS-001"
    defined_for = StateCode.AR

    def formula(spm_unit, period, parameters):
        income_eligible = spm_unit("ar_tea_income_eligible", period)
        resources_eligible = spm_unit("ar_tea_resources_eligible", period)
        demographic_eligible = spm_unit("is_demographic_tanf_eligible", period)
        # Must have at least one U.S. citizen or qualified immigrant
        immigration_eligible = (
            add(spm_unit, period, ["is_citizen_or_legal_immigrant"]) > 0
        )
        return (
            income_eligible
            & resources_eligible
            & demographic_eligible
            & immigration_eligible
        )



from policyengine_us.model_api import *


class fl_tca_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "Florida TCA gross income eligible"
    definition_period = MONTH
    reference = (
        "https://www.law.cornell.edu/regulations/florida/Fla-Admin-Code-Ann-R-65A-4-220",
        "https://www.myflfamilies.com/services/public-assistance/temporary-cash-assistance",
    )
    defined_for = StateCode.FL

    def formula(spm_unit, period, parameters):
        # Per DCF 2620.0109.01: Gross income test - NO disregards applied
        # "The standard earned income disregard, 200 and 1/2 which includes
        # the standard earned income disregard of $90, is not deducted in this test."
        p = parameters(period).gov.states.fl.dcf.tanf.income.gross_test

        gross_income = spm_unit("fl_tca_gross_income", period)

        # Compare to 185% FPL (annual FPG converted to monthly)
        fpg = spm_unit("spm_unit_fpg", period)
        gross_income_limit = fpg * p.gross_income_limit_rate

        return gross_income < gross_income_limit



from policyengine_us.model_api import *


class nj_wfnj_gross_income_eligible(Variable):
    value_type = bool
    entity = SPMUnit
    label = "New Jersey WFNJ gross income eligible"
    definition_period = MONTH
    defined_for = StateCode.NJ
    reference = (
        "https://www.law.cornell.edu/regulations/new-jersey/N-J-A-C-10-90-3-3",
    )

    def formula(spm_unit, period, parameters):
        p = parameters(period).gov.states.nj.njdhs.wfnj
        gross_income = spm_unit("nj_wfnj_gross_income", period)
        payment_levels = spm_unit("nj_wfnj_payment_levels", period)
        max_allowable = np.round(payment_levels * p.income.limit)
        return gross_income <= max_allowable



from policyengine_us.model_api import *


class ut_fep_gross_income(Variable):
    value_type = float
    entity = SPMUnit
    label = "Utah Family Employment Program gross income"
    unit = USD
    definition_period = MONTH
    reference = "https://www.law.cornell.edu/regulations/utah/Utah-Admin-Code-R986-200-239"
    defined_for = StateCode.UT

    adds = [
        "tanf_gross_earned_income",
        "tanf_gross_unearned_income",
    ]



from policyengine_us.model_api import *


class il_scretd_age_eligible(Variable):
    value_type = bool
    entity = Person
    label = "Illinois Senior Citizens Real Estate Tax Deferral age eligibility"
    definition_period = YEAR
    defined_for = StateCode.IL
    reference = "https://www.ilga.gov/legislation/ilcs/ilcs3.asp?ActID=1454&ChapterID=31"

    def formula(person, period, parameters):
        p = parameters(period).gov.states.il.idor.scretd
        age = person("age", period)
        return age >= p.age_threshold
